/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.admin;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;

import javax.swing.*;
import javax.swing.border.TitledBorder;

import org.apache.commons.lang.StringUtils;

import com.funambol.admin.AdminException;
import com.funambol.admin.ui.ConnectorManagementPanel;
import com.funambol.email.util.Def;

import java.awt.Dimension;


/**
 * This class implements the connector configuration panel
 * for a Email Connector component
 *
 * @version $Id: EmailConnectorConfigPanel.java,v 1.10 2006/06/14 08:30:45 gbmiglia Exp $
 */
public class EmailConnectorConfigPanel
        extends ConnectorManagementPanel
        implements Serializable, ActionListener

{

    // --------------------------------------------------------------- Constants

    public static final String NAME_ALLOWED_CHARS =
            "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_.";

    // ------------------------------------------------------------ Private data

    private JPanel serverPanel                 = new JPanel();
    private JPanel funambolPanel               = new JPanel();
    private JPanel propertyPanel               = new JPanel();

    private JLabel hostLabel                   = new JLabel();
    private JTextField hostValue               = new JTextField();
    private JLabel protocolLabel               = new JLabel();
    private JComboBox protocolValue            = new JComboBox();
    private JLabel portLabel                   = new JLabel();
    private JTextField portValue               = new JTextField();
    private JLabel sslLabel                    = new JLabel();
    private JCheckBox sslValue                 = new JCheckBox();
    private JLabel keystoreLabel               = new JLabel();
    private JTextField keystoreValue           = new JTextField();
    private JLabel keystorePassPhraseLabel     = new JLabel();
    private JTextField keystorePassPhraseValue = new JTextField();

    private JLabel dataSourceLabel             = new JLabel();
    private JTextField dataSourceValue         = new JTextField();

    private JLabel filterLabel                 = new JLabel();
    private JCheckBox filterValue              = new JCheckBox();

    private JButton confirmButton              = new JButton();

    // ------------------------------------------------------------ Constructors


    /**
     *<p>Default constructor.</p>
     */
    public EmailConnectorConfigPanel() {
        init();
    }

    // --------------------------------------------------------- Private methods

    /**
     * Create the panel
     */
    private void init() {

        JLabel title = new JLabel();
        setLayout(null);

        title.setFont(titlePanelFont);
        title.setText("Funambol Email Connector");
        title.setBounds(new Rectangle(14, 5, 316, 28));
        title.setAlignmentX(SwingConstants.CENTER);
        title.setBorder(new TitledBorder(""));

        serverPanel.setLayout(null);
        serverPanel.setBorder(new TitledBorder("Officer Configuration"));

        hostLabel.setText("Mail Server:");
        serverPanel.add(hostLabel);
        hostLabel.setBounds(10, 20, 116, 15);
        serverPanel.add(hostValue);
        hostValue.setBounds(150, 20, 220, 19);
        hostValue.setFont(defaultFont);

        protocolLabel.setText("Protocol:");
        serverPanel.add(protocolLabel);
        protocolLabel.setBounds(10, 40, 131, 15);
        serverPanel.add(protocolValue);
        protocolValue.setBounds(150, 40, 60, 19);
        protocolValue.setFont(defaultFont);
        protocolValue.addItem(Def.PROTOCOL_POP3);
        protocolValue.addItem(Def.PROTOCOL_IMAP);
        protocolValue.addActionListener(this);

        portLabel.setText("Port:");
        serverPanel.add(portLabel);
        portLabel.setBounds(10, 61, 131, 15);
        serverPanel.add(portValue);
        portValue.setBounds(150, 61, 220, 19);
        portValue.setFont(defaultFont);

        sslLabel.setText("SSL:");
        serverPanel.add(sslLabel);
        sslLabel.setBounds(new Rectangle(10, 80, 150, 19));
        sslValue.setSelected(false);
        serverPanel.add(sslValue);
        sslValue.setBounds(146, 80, 220, 19);
        sslValue.addActionListener(this);

        keystoreLabel.setText("Keystore Path:");
        serverPanel.add(keystoreLabel);
        keystoreLabel.setBounds(10, 100, 131, 15);
        keystoreLabel.setEnabled(false);
        serverPanel.add(keystoreValue);
        keystoreValue.setBounds(150, 100, 220, 19);
        keystoreValue.setFont(defaultFont);
        keystoreValue.setEnabled(false);

        keystorePassPhraseLabel.setText("Keystore Passphrase:");
        serverPanel.add(keystorePassPhraseLabel);
        keystorePassPhraseLabel.setBounds(10, 120, 131, 15);
        keystorePassPhraseLabel.setEnabled(false);
        serverPanel.add(keystorePassPhraseValue);
        keystorePassPhraseValue.setBounds(150, 120, 220, 19);
        keystorePassPhraseValue.setFont(defaultFont);
        keystorePassPhraseValue.setEnabled(false);

        add(serverPanel);
        serverPanel.setBounds(10, 50, 380, 160);

        funambolPanel.setLayout(null);
        funambolPanel.setBorder(new TitledBorder("Funambol Configuration"));

        dataSourceLabel.setText("Funambol DataSource:");
        funambolPanel.add(dataSourceLabel);
        dataSourceLabel.setBounds(10, 20, 131, 15);
        funambolPanel.add(dataSourceValue);
        dataSourceValue.setBounds(150, 20, 220, 19);
        dataSourceValue.setFont(defaultFont);

        add(funambolPanel);
        funambolPanel.setBounds(10, 220, 380, 50);

        propertyPanel.setLayout(null);
        propertyPanel.setBorder(new TitledBorder("Property Configuration"));

        filterLabel.setText("Filter Activation: ");
        propertyPanel.add(filterLabel);
        filterLabel.setBounds(new Rectangle(10, 20, 150, 19));
        filterValue.setSelected(true);
        propertyPanel.add(filterValue);
        filterValue.setBounds(146, 20, 220, 19);

        add(propertyPanel);
        propertyPanel.setBounds(10, 280, 380, 50);

        confirmButton.setFont(defaultFont);
        confirmButton.setText("Save");
        confirmButton.setBounds(160, 350, 70, 25);

        confirmButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                try {
                    validateValues();
                    getValues();
                    EmailConnectorConfigPanel.this.actionPerformed(
                            new ActionEvent(EmailConnectorConfigPanel.this,
                                    ACTION_EVENT_UPDATE,
                                    event.getActionCommand()));
                } catch (Exception e) {
                    notifyError(new AdminException(e.getMessage()));
                }
            }
        });

        add(confirmButton, null);

        Component[] components = getComponents();
        for (int i = 0; (components != null) && (i < components.length); ++i) {
            components[i].setFont(defaultFont);
        }
        add(title);
    }

    /**
     * Set preferredSize of the panel
     *
     * @return preferredSize of the panel
     */
    public Dimension getPreferredSize() {
        return new Dimension(525, 380);
    }

    /**
     *
     */
    public void updateForm() {

        EmailConnectorConfig emailConf = (EmailConnectorConfig) getConfiguration();

        hostValue.setText(emailConf.getHost());

        protocolValue.setSelectedItem(emailConf.getProtocol());

        portValue.setText("" + emailConf.getPort());

        sslValue.setSelected(emailConf.getSsl());


        keystoreValue.setText(emailConf.getKeystorePath() == null ? "" : emailConf.getKeystorePath());
        keystorePassPhraseValue.setText(emailConf.getKeystorePassPhrase() == null ?
                "" : emailConf.getKeystorePassPhrase());
        keystoreLabel.setEnabled(sslValue.isSelected());
        keystoreValue.setEnabled(sslValue.isSelected());
        keystorePassPhraseLabel.setEnabled(sslValue.isSelected());
        keystorePassPhraseValue.setEnabled(sslValue.isSelected());

        dataSourceValue.setText(emailConf.getDataSource());

        filterValue.setSelected(emailConf.getFilter());

    }

    public void actionPerformed(ActionEvent actionEvent) {
        if (actionEvent.getSource().equals(this.protocolValue)) {
            if (Def.PROTOCOL_IMAP.equals(protocolValue.getSelectedItem())) {
                portValue.setText("" + Def.PROTOCOL_IMAP_PORT);
            }
            if (Def.PROTOCOL_POP3.equals(protocolValue.getSelectedItem())) {
                portValue.setText("" + Def.PROTOCOL_POP3_PORT);
            }
            return;
        }
        if (actionEvent.getSource().equals(sslValue)) {
            keystoreLabel.setEnabled(sslValue.isSelected());
            keystoreValue.setEnabled(sslValue.isSelected());
            keystorePassPhraseLabel.setEnabled(sslValue.isSelected());
            keystorePassPhraseValue.setEnabled(sslValue.isSelected());
            return;
        }
        super.actionPerformed(actionEvent);
    }

    // ----------------------------------------------------------- Private methods

    /**
     * Checks if the values provided by the user are all valid. In caso of errors,
     * a IllegalArgumentException is thrown.
     *
     * @throws IllegalArgumentException if:
     *                                  <ul>
     *                                  <li>anyvalue is not in the form <host>:<port>
     *                                  </ul>
     */
    private void validateValues() throws IllegalArgumentException {
        String value;

        // in host
        value = hostValue.getText().trim();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException("Field 'Host' cannot be empty");
        }

        // protocol
        value = protocolValue.getSelectedItem().toString().trim();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException("Field 'Protocol' cannot be empty");
        }

        // port
        value = portValue.getText().trim();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException("Field 'Port' cannot be empty");
        }
        try {
            Integer.parseInt(value);
        } catch (NumberFormatException e) {
            throw new IllegalArgumentException("Field 'Port' must be numeric");
        }

        // datasource
        value = dataSourceValue.getText().trim();
        if (StringUtils.isEmpty(value)) {
            throw new IllegalArgumentException("Field 'Sync4j DataSource' cannot be empty");
        }


    }

    /**
     * Set properties with the values provided by the user.
     */
    private void getValues() {

        EmailConnectorConfig emailConf = (EmailConnectorConfig) getConfiguration();

        emailConf.setHost(hostValue.getText().trim());

        emailConf.setProtocol(protocolValue.getSelectedItem().toString().trim());

        emailConf.setPort(Integer.parseInt(portValue.getText().trim()));

        if (sslValue.isSelected()) {
            emailConf.setSsl(true);
            emailConf.setKeystorePath(keystoreValue.getText().trim());
            emailConf.setKeystorePassPhrase(keystorePassPhraseValue.getText().trim());
        } else {
            emailConf.setSsl(false);
            emailConf.setKeystorePath(null);
            emailConf.setKeystorePath(null);
        }

        emailConf.setDataSource(dataSourceValue.getText().trim());

        if (filterValue.isSelected()) {
            emailConf.setFilter(true);
        } else {
            emailConf.setFilter(false);
        }

    }

}
