/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.engine.source;

import java.io.Serializable;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.sql.Timestamp;

import com.funambol.framework.engine.SyncItem;
import com.funambol.framework.engine.SyncItemKey;
import com.funambol.framework.engine.source.FilterableSyncSource;
import com.funambol.framework.engine.source.SyncSourceException;
import com.funambol.framework.security.Sync4jPrincipal;
import com.funambol.framework.engine.SyncItemState;
import com.funambol.framework.tools.beans.LazyInitBean;
import com.funambol.framework.tools.beans.BeanInitializationException;

import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.items.manager.PopEntityManager;
import com.funambol.email.engine.cache.CrcSyncItemInfo;
import com.funambol.email.items.model.SendResult;
import com.funambol.email.pdi.mail.Email;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.EntityException;
import com.funambol.email.exception.SendingException;
import com.funambol.email.pdi.folder.Folder;
import com.funambol.framework.engine.source.SyncContext;


/**
 * This class implements common methods of EmailPOPSyncSource
 * <p/>
 * <p/>
 * version $Id: PopEmailSyncSource.java,v 1.30 2006/06/25 14:39:11 gbmiglia Exp $
 */
public final class PopEmailSyncSource
        extends EmailSyncSource
        implements FilterableSyncSource, Serializable, LazyInitBean {

    // --------------------------------------------------------------- Constants

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // -------------------------------------------------------------- Properties

    protected PopEntityManager pem = null;

    // ---------------------------------------------------------- Public Methods

    {
        this.setInPort("" + Def.PROTOCOL_POP3_PORT);
    }

    
    /**
     *
     */
    public void beginSync(SyncContext context) throws SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("POP EmailSyncSource - Start beginSync [" + sourceURI + "]");
            log.finest("call Generic Email SyncSource");
        }
        super.beginSync(context);
    }    
    
    
    /**
     * Init the syncSource
     *
     * @throws BeanInitializationException
     */
    public void init() throws BeanInitializationException {
        super.init();
    }


    /**
     * @param syncItem SyncItem
     * @return boolean
     * @throws SyncSourceException
     */
    public boolean isSyncItemInFilterClause(SyncItem syncItem)
            throws SyncSourceException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("POP3 isSyncItemInFilterClause(syncItem)");
        }
        
        String content;
        try {
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " +
                    syncItem, e);
        }

        boolean isemail = isEmailFromContent(content);

        boolean isInFilter = false;
        
        if (isemail) {
            isInFilter = isEmailInFilter(content, syncItem);
        } else {
            isInFilter = isFolderInFilter(content, syncItem);
        }

        return isInFilter;
        
    }

    /**
     * @param syncItemKey SyncItemKey
     * @return boolean
     * @throws SyncSourceException
     */
    public boolean isSyncItemInFilterClause(SyncItemKey syncItemKey)
            throws SyncSourceException {
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("POP3 isSyncItemInFilterClause(syncItemKey)");
        }
        
        String key  = (String) syncItemKey.getKeyValue();
        String FID  = Utility.getIdBeforeSeparator(key);
        String FMID = Utility.getIdAfterSeparator(key);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("isSyncItemInFilter in FID: " + FID + " with FMID: " + FMID);
        }

        boolean isemail = isEmailFromKey(FID, FMID);
        
        boolean isInFilter = false;
        
        if (isemail) {
            isInFilter = isEmailInFilter(key);
        } else {
            isInFilter = isFolderInFilter(key);
        }

        return isInFilter;
    }

    
    /**
     *
     * check if the message is included in the filter clause
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private boolean isFolderInFilter(String key) throws SyncSourceException {
        boolean isInFilter = false;       
        return isInFilter;

    }
    
    
    /**
     *
     * check if the message is included in the filter clause
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private boolean isFolderInFilter(String content, SyncItem syncItem) 
     throws SyncSourceException {
        boolean isInFilter = false;       
        return isInFilter;

    }
    

    /**
     * @param operation  String
     * @param statusCode int
     * @param keys       SyncItemKey[]
     */
    public void setOperationStatus(String operation,
                                   int statusCode,
                                   SyncItemKey[] keys) {

        if (log.isLoggable(Level.FINEST)) {
            StringBuffer message = new StringBuffer("Received status code '");
            message.append(statusCode).append("' for a '").append(operation).append("'").
                    append(" for this items: ");

            for (int i = 0; i < keys.length; i++) {
                message.append("\n- ").append(keys[i].getKeyAsString());
            }
            log.finest(message.toString());
        }
    }


    /**
     * @param syncItemKey SyncItemKey
     * @param s           Timestamp
     * @param softDelete  boolean
     * @throws SyncSourceException
     */
    public void removeSyncItem(SyncItemKey syncItemKey,
                               Timestamp s,
                               boolean softDelete)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("remove item softDelete: " + softDelete);
        }

        if (softDelete) {
            // nothing
        } else {
            // if false
            removeItem(syncItemKey);
        }

    }

    /**
     * @param syncItemKey SyncItemKey
     * @return SyncItem
     * @throws SyncSourceException
     * @see com.funambol.framework.engine.source.SyncSource
     */
    public SyncItem getSyncItemFromId(SyncItemKey syncItemKey)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemFromId start");
        }

        String key = (String) syncItemKey.getKeyValue();

        String parentId = Utility.getIdBeforeSeparator(key);
        String objectId = Utility.getIdAfterSeparator(key);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("get item - FID: " + parentId + "; FMID: " + objectId);
        }

        SyncItem syncItem;

        boolean isEmail = isEmailFromKey(parentId, objectId);

        if (isEmail) {
            syncItem = getEmailFromId(syncItemKey);
        } else {
            syncItem = getFolderFromId(syncItemKey);
        }

        return syncItem;

    }

    /**
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getAllSyncItemKeys() throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItems start ");
        }

        String[]   allIds;
        String[]   mailIds;
        String[]   folderIds;

        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {

            this.pem = getPopEntityManager();

            mailIds = this.pem.getAllEmails(filter,
                    this.sourceURI,
                    pID);

            folderIds = this.pem.getAllFolders();

        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting manager layer: " +
                    e.getMessage(), e);
        } catch (Exception e) {
            throw new SyncSourceException("Error reading getAllSyncItems : " +
                    e.getMessage(), e);
        }

        allIds = Utility.mergeArray(mailIds, folderIds, null, null, null);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Pop getAllSyncItems() end");
        }

        SyncItemKey[] syncItemKeys = null;
        if (allIds != null) {
            syncItemKeys = new SyncItemKey[allIds.length];
            for (int i = 0; i < allIds.length; i++) {
                syncItemKeys[i] = new SyncItemKey(allIds[i]);
            }
        }

        return syncItemKeys;
    }

    /**
     * @return SyncItemInfo[]
     * @throws SyncSourceException
     */
    public CrcSyncItemInfo[] getAllSyncItemInfo() throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemInfo start");
        }

        CrcSyncItemInfo[] allInfo;
        CrcSyncItemInfo[] emailInfo;
        CrcSyncItemInfo[] folderInfo;

        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {
            this.pem = getPopEntityManager();

            emailInfo = this.pem.getAllEmailInfo(this.filter,
                    this.sourceURI,
                    pID);

            folderInfo = this.pem.getAllFolderInfo();

        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting mails: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error reading folders: " +
                    e.getMessage(), e);
        }

        // add main folders
        allInfo = Utility.mergeArray(emailInfo, folderInfo,
                null, null, null);


        return allInfo;
    }


    /**
     * @throws SyncSourceException
     */
    public void clearServerInfo() throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("cleanServerInfo start");
        }

        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {

            this.pem = getPopEntityManager();

            this.pem.clearAllItems(this.sourceURI, pID);

        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting mails: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error deleting all emails/folders: " +
                    e.getMessage(), e);
        }

    }

    /**
     * @param syncItem SyncItem
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getSyncItemKeysFromTwin(SyncItem syncItem) throws SyncSourceException {

        String content;
        try {
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " + syncItem, e);
        }

        boolean isemail = isEmailFromContent(content);

        SyncItemKey[] syncItemKeys;

        if (isemail) {
            syncItemKeys = getEmailFromTwin(content, syncItem);
        } else {
            syncItemKeys = getFolderFromTwin(content, syncItem);
        }

        return syncItemKeys;
    }

    /**
     * This method can only send the mail but not
     * append the mail in the outbox and sent folder.
     *
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem addSyncItem(SyncItem syncItem) throws SyncSourceException {

        String content;
        try {
            content = Utility.getContentFromSyncItem(syncItem);
        }
        catch (Exception e) {
            throw new SyncSourceException("Error getting the item " + syncItem, e);
        }

        boolean isEmail = isEmailFromContent(content);

        if (isEmail) {
            syncItem = addEmail(content, syncItem);
        } else {
            syncItem = addFolder(syncItem);
        }

        return syncItem;

    }


    /**
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem updateSyncItem(SyncItem syncItem) throws SyncSourceException {

        String content;
        try {
            content = Utility.getContentFromSyncItem(syncItem);
        } catch (Exception e) {
            throw new SyncSourceException("Error getting the item " + syncItem, e);
        }

        boolean isEmail = isEmailFromContent(content);

        String key = syncItem.getKey().getKeyAsString();

        String parentId = Utility.getIdBeforeSeparator(key);
        String objectId = Utility.getIdAfterSeparator(key);

        if (isEmail) {
            syncItem = updateEmail(parentId, objectId, content, syncItem);
        } else {
            syncItem = updateFolder(parentId, objectId, content, syncItem);
        }
             
        return syncItem;

    }

    // --------------------------------------------------------- Private Methods


    /**
     * the key is composed by: parentId/objId (mailid or folderid)
     * The key is created by the connector and it always
     * creates the syncItemKey using parentId and mailId
     *
     * @param syncItemKey SyncItemKey
     * @throws SyncSourceException
     */
    private void removeItem(SyncItemKey syncItemKey)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("EmailIMAPSyncSource removeSyncItem [" + sourceURI + "]");
        }

        String key = syncItemKey.getKeyAsString();

        String parentId = Utility.getIdBeforeSeparator(key);
        String objectId = Utility.getIdAfterSeparator(key);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("remove item in FID: " + parentId + "; FMID: " + objectId);
        }

        boolean isEmail = isEmailFromKey(parentId, objectId);

        if (isEmail) {
            removeEmail(syncItemKey);
        }

    }


    /**
     *
     * This method can only send the mail but it cannot
     * append the mail in the folders (i.e. inbox, outbox and sent).
     * If the folder ID is different form OUTBOX_ID,
     * the method returns the input syncItem.
     * <p/>
     * We can't insert an email using POP3 protocol, in this case
     * we save the mail in the SERVER LOCAL DB not in the MAIL SERVER DB
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem addEmail(String content, SyncItem syncItem)
            throws SyncSourceException {

        String[] keys = Utility.getIdBeforeAndAfter(syncItem);
        String FID    = keys[0];
        String FMID   = keys[1];

        if (log.isLoggable(Level.FINEST)) {
            log.finest("addEmail folder: " + FID + " luid: " + FMID);
        }

        Email ctmpNew       = null;
        Email insertedEmail = null;
        String GUID         = null;
        long  pID           = ((Sync4jPrincipal) this.principal).getId();
        
        try {
            ctmpNew = getFoundationMailFromXML(content);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                                          e.getMessage(), e);
        }

        if (FID.equalsIgnoreCase(Def.FOLDER_OUTBOX_ID)) {
            
            try {
                
                this.pem = getPopEntityManager();
                
                //add and send the email
                insertedEmail = this.pem.addEmail(FID, ctmpNew, idSentSpace,
                                                  this.sourceURI, pID);
                
                if (insertedEmail != null) {
                    // message add and sent without error, set to true the result
                    GUID = (String) insertedEmail.getUID().getPropertyValue();
                    String idtmp = Utility.convertOutboxIDToSentID(GUID);
                    SendResult sr = new SendResult(idtmp, FMID, true, true);
                    itemsSent.add(sr);
                }
                
            } catch (EmailAccessException e) {
                throw new SyncSourceException("Error connecting manager layer: " +
                                              e.getMessage(), e);
            } catch (EntityException e) {
                String errorMessage = "Error setting the item: " + syncItem;
                if (log.isLoggable(Level.SEVERE)) {
                    log.severe(errorMessage);
                }
                throw new SyncSourceException(errorMessage, e);
            } catch (SendingException e) {
                // message sent with error, set to false the result
                SendResult sr = new SendResult(GUID, FMID, false, true);
                itemsSent.add(sr);
                throw new SyncSourceException(e.getMessage(), 500);                
            }
          
        }

        // create the output syncItem
        try {
            if (insertedEmail != null) {
                syncItem = createSyncItem(insertedEmail, SyncItemState.NEW);
            }
        } catch (Exception e) {
            throw new SyncSourceException("Error creating SyncItem: " + syncItem, e);
        }

        return syncItem;
    }


    /**
     * Create PopEntityManager istance (if not already exist)
     *
     * @return PopEntityManager
     * @throws EmailAccessException
     */
    private PopEntityManager getPopEntityManager() throws EmailAccessException {
        if (this.pem == null) {
            this.pem = new PopEntityManager(this.mswf, this.jndiDataSourceName);
        }
        return this.pem;
    }


    /**
     * the email syncItemKey is composed by
     * FID + separator + FMID
     * The FID must be converted in the fullPath
     *
     * @param syncItemKey SyncItemKey
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem getEmailFromId(SyncItemKey syncItemKey)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getEmailFromId start");
        }

        SyncItem syncItem = null;
        Email    mail     = null;

        String GUID = syncItemKey.getKeyAsString();

        long pID = ((Sync4jPrincipal) this.principal).getId();

        char status = getSyncItemStateFromId(syncItemKey);

        try {
            this.pem = getPopEntityManager();
            mail = this.pem.getEmailFromUID(this.filter, GUID, status,
                                            this.sourceURI, pID);
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting POP store: " +
                                          e.getMessage(), e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error in getting Mail " +
                                          e.getMessage(), e);
        }

        if (mail != null) {
            try {
                syncItem = createSyncItem(mail, status);
            } catch (Exception e) {
                throw new SyncSourceException("Error in creating SyncItem ", e);
            }
        }

        return syncItem;

    }

    /**
     * @param syncItemKey SyncItemKey
     * @return SyncItem
     * @throws SyncSourceException
     * @see com.funambol.framework.engine.source.SyncSource
     */
    private SyncItem getFolderFromId(SyncItemKey syncItemKey)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemFromId start");
        }

        SyncItem syncItem = null;
        Folder folder;

        String id = syncItemKey.getKeyAsString();
        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {
            this.pem = getPopEntityManager();
            folder = this.pem.getFolderFromUID(id, this.sourceURI, pID);
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting email: " +
                    e.getMessage(), e);
        } catch (Exception e) {
            throw new SyncSourceException("Error in getting Folder ", e);
        }

        // get item status
        char status = getSyncItemStateFromId(syncItemKey);

        if (folder != null) {
            try {
                syncItem = createSyncItem(folder, status);
            } catch (Exception e) {
                throw new SyncSourceException("Error in creating SyncItem ", e);
            }
        }

        return syncItem;

    }

    /**
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    private SyncItemKey[] getEmailFromTwin(String content,
                                           SyncItem syncItem)
      throws SyncSourceException {

        String key  = (String) syncItem.getKey().getKeyValue();
        String FID  = Utility.getIdBeforeSeparator(key);
        String FMID = Utility.getIdAfterSeparator(key);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getEmailFromTwin with UID: " + key +
                       " FMID " + FMID + " FID " + FID);
        }

        long pID = ((Sync4jPrincipal) this.principal).getId();

        String GUID = null;

        // search only in inbox folder
        if (FID.equals(Def.FOLDER_INBOX_ID)) {

            Email email;

            try {
                email = getFoundationMailFromXML(content);
            } catch (EntityException e) {
                throw new SyncSourceException("Error converting foundation: " +
                        e.getMessage(), e);
            }

            try {
                this.pem = getPopEntityManager();
                GUID = this.pem.getEmailFromClause(FID, FMID, email,
                                                   this.sourceURI, pID);
            } catch (EmailAccessException e) {
                throw new SyncSourceException("Error opening EmailPopManager ", e);
            } catch (EntityException e) {
                throw new SyncSourceException("Error getting twin", e);
            }
        }

        // converts Email in SyncItem
        SyncItemKey[] syncItemKeys;

        if (GUID != null) {
            syncItemKeys = new SyncItemKey[1];
            syncItemKeys[0] = new SyncItemKey(GUID);
        } else {
            syncItemKeys = new SyncItemKey[0];
        }

        return syncItemKeys;

    }

    /**
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    private SyncItemKey[] getFolderFromTwin(String content,
                                            SyncItem syncItem)
      throws SyncSourceException {


        SyncItemKey[] syncItemKeys;

        String key = (String) syncItem.getKey().getKeyValue();

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getSyncItemKeysFromTwin with UID: " + key);
        }

        Folder ctmpNew;
        try {
            ctmpNew = getFoundationFolderFromXML(content);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        // insert folder
        com.funambol.email.pdi.folder.Folder insertedFolder;

        String name = ctmpNew.getName().getPropertyValueAsString();
        String parentId = syncItem.getParentKey().getKeyAsString();

        if (parentId.equalsIgnoreCase("/")) {
            parentId = Def.FOLDER_ROOT_ID;
        }

        long pID = ((Sync4jPrincipal) this.principal).getId();

        try {

            this.pem = getPopEntityManager();

            insertedFolder = this.pem.getFolderFromName(name, parentId,
                    this.sourceURI, pID);

            if (insertedFolder != null) {
                syncItemKeys = new SyncItemKey [1];
                String GUID = insertedFolder.getUID().getPropertyValueAsString();
                syncItemKeys[0] = new SyncItemKey(GUID);
            } else {
                syncItemKeys = new SyncItemKey[0];
            }

        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting manager layer: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            String errorMessage = "Error setting the item: " + syncItem;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new SyncSourceException(errorMessage, e);
        }

        return syncItemKeys;
    }


    /**
     * @param syncItemKey
     * @throws SyncSourceException
     */
    private void removeEmail(SyncItemKey syncItemKey)
            throws SyncSourceException {

        String key      = syncItemKey.getKeyAsString();
        String parentId = Utility.getIdBeforeSeparator(key);
        String mailId   = Utility.getIdAfterSeparator(key);
        long pID        = ((Sync4jPrincipal) this.principal).getId();

        try {
            this.pem = getPopEntityManager();
            this.pem.removeEmail(parentId, mailId, this.sourceURI, pID);
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting email: " +
                    e.getMessage(), e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error removing items", e);
        }

    }

    /**
     *
     * check if the message is included in the filter clause
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private boolean isEmailInFilter(String content, SyncItem syncItem) 
      throws SyncSourceException {

        boolean isInFilter = false;
        
        // get SyncItem UID
        String key    = syncItem.getKey().getKeyAsString();
        String[] keys = Utility.getIdBeforeAndAfter(syncItem);
        String FID    = keys[0];
        String FMID   = keys[1];

        long pID = ((Sync4jPrincipal) this.principal).getId();

        Email email;
        try {            
            email = getFoundationMailFromXML(content);
            this.pem   = getPopEntityManager();            
            isInFilter = this.pem.isEmailInFilter(filter, FID, key, email);            
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error accessing MailBox", e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error matching mail with filter ", e);
        }
                
        return isInFilter;

    }  
    
    
    /**
     *
     * check if the message is included in the filter clause
     *
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private boolean isEmailInFilter(String key) throws SyncSourceException {

        boolean isInFilter = false;
        
        String parentId = Utility.getIdBeforeSeparator(key);
        String mailId   = Utility.getIdAfterSeparator(key);

        long pID = ((Sync4jPrincipal) this.principal).getId();
                
        try {            
            this.pem   = getPopEntityManager();          
            isInFilter = this.pem.isEmailInFilter(filter, key, parentId, mailId, 
                                                  this.sourceURI, pID);
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error connecting manager layer: " +
                                          e.getMessage(), e);
        } catch (EntityException e) {
            throw new SyncSourceException("Error checking the item: " + 
                                          e.getMessage() , e);
        }

        return isInFilter;
    }    
    
    
    /**
     * the pop3-based syncsource can't update the item 
     * in the mail server. In this case this method must 
     * return the item for internal ds-server purpose
     *
     * @param parentId  String
     * @param mailId  String
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem updateEmail(String parentId,
                                 String mailId,
                                 String content,
                                 SyncItem syncItem)
      throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateSyncItem FID " + parentId + " FMID " + mailId);
        }
        
        Email emailOut;
        long pID = ((Sync4jPrincipal) this.principal).getId();
        
        Email emailNew;
        try {
            emailNew = getFoundationMailFromXML(content);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        try {
            this.pem = getPopEntityManager();
            emailOut = this.pem.updateEmail(parentId, mailId, emailNew, 
                                            this.sourceURI, pID);            
        } catch (Exception e) {
            throw new SyncSourceException("Error setting the item: ", e);
        }

        // create the output syncItem
        try {
            if (emailOut != null) {
                syncItem = createSyncItem(emailOut, SyncItemState.UPDATED);
            }
        } catch (Exception e) {
            throw new SyncSourceException("Error creating SyncItem: " + syncItem, e);
        }
        
        return syncItem;
    }

    /**
     * @param content  String
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem updateFolder(String parentId,
                                  String folderId,
                                  String content,
                                  SyncItem syncItem)

            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("updateFolder FID " + parentId + " and FMID " + folderId);
        }
        
        Folder folderNew;
        try {
            folderNew = getFoundationFolderFromXML(content);
        } catch (EntityException e) {
            throw new SyncSourceException("Error converting foundation: " +
                    e.getMessage(), e);
        }

        Folder folderOut;
        try {
            this.pem = getPopEntityManager();
            folderOut = this.pem.updateFolder(parentId, folderId, folderNew);
        } catch (Exception e) {
            String errorMessage = "Error setting the item: " + syncItem;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new SyncSourceException(errorMessage, e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest(" end update folder. start create SyncItem");
        }

        // create the output syncItem
        try {
            if (folderOut != null) {
                syncItem = createSyncItem(folderOut, SyncItemState.UPDATED);
            }
        } catch (Exception e) {
            throw new SyncSourceException("Error creating SyncItem: " + syncItem, e);
        }

        return syncItem;

    }

    /**
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    private SyncItem addFolder(SyncItem syncItem) throws SyncSourceException {
        return syncItem;
    }

}
