/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.inboxlistener.console;


import java.util.List;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;

import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.exception.InboxListenerConfigException;
import com.funambol.email.inboxlistener.engine.DBAccount;
import com.funambol.email.inboxlistener.engine.MailboxAccount;
import com.funambol.email.inboxlistener.admin.InboxListenerConfig;

/**
 * simple admin console
 *
 * @version $Id: Menu.java,v 1.6 2006/05/31 12:14:12 gbmiglia Exp $
 */
public class Menu {

  private DBAccount dba     = null;
  private MenuDAO   menuDAO = null;

  /**
   *
   */
  public Menu() {
     go();
  }

  /**
   *
   */
  public static void main(String[] args) {
     Menu m = new Menu();
  }


  /**
   *
   */
  public void go(){

     // load DataBase Properties
     String dbname        = "";
     String jdbcDriver    = "";
     String jdbcUrl       = "";
     String jdbcUser      = "";
     String jdbcPassword  = "";
     int    reloadingTime = 0;
     try {
        jdbcDriver    = InboxListenerConfig.getConfigInstance().getJdbcDriver();
        jdbcUrl       = InboxListenerConfig.getConfigInstance().getJdbcUrl();
        jdbcUser      = InboxListenerConfig.getConfigInstance().getJdbcUser();
        jdbcPassword  = InboxListenerConfig.getConfigInstance().getJdbcPassword();
        reloadingTime = InboxListenerConfig.getConfigInstance().getReloadingTime();
        dba          = new DBAccount(jdbcDriver,
                                     jdbcUrl,
                                     jdbcUser,
                                     jdbcPassword,
                                     reloadingTime);
     } catch (InboxListenerConfigException e) {
       e.printStackTrace();
     }

     //
     menuDAO = new MenuDAO(dba);

     try {
        // clean display
        for (int p=0;p<50;p++)
          System.out.println("");

        int option = -1;

        while (true) {
          option = mainMenu();
          if (option == 0) {
            break;
          }
        }
     }
     catch(java.lang.Exception e) {
        e.printStackTrace();
     }

  }



  /**
   *
   */
  public int mainMenu()
  {
    String option = null;
    int value = -1;

    try {
       System.out.println("");
       System.out.println("");
       System.out.println(" INBOX LISTENER ACCOUNT CONSOLE");
       System.out.println("");
       System.out.println(" 0   quit");
       System.out.println(" 1   list accounts");
       System.out.println(" 2   insert account");
       System.out.println(" 3   delete account");
       System.out.println(" ");

       value = readInteger();

       switch (value) {
        case 1: list(); break;
        case 2: insert(); break;
        case 3: delete(); break;
        case 0: break;
        default: System.out.println("error");
         return 0;
       }
      } catch(java.lang.Exception e){
        e.printStackTrace();
        return 0;
      }
      return value;
   }


  //------------------------------------------------------------ PRIVATE METHODS


  /**
   *
   */
  private void list() throws InboxListenerException {
     try {
        List l = menuDAO.getAccounts();
        if (l != null) {
          if (l.size() == 0){
            System.out.println("No accounts found");
          }
          for (int i=0; i < l.size(); i++){
            System.out.println(((MailboxAccount)l.get(i)).toString());
          }
        } else {
          System.out.println("No accounts found");
        }
     } catch (Exception ex) {
        throw new InboxListenerException("Error reading accounts (" +
                                          ex.getMessage() + ")");
     }
  }

  /**
   *
   */
  private void insert() throws InboxListenerException {
     try {
       System.out.print("username: ");
       String username = readValue();
       System.out.print("password: ");
       String password = readValue();
       System.out.print("listener id (number): ");
       String listenerID = readValue();

       MailboxAccount ma = new MailboxAccount(username,
                                              password,
                                              true,
                                              listenerID);

       menuDAO.insertAccount(ma);

     } catch (Exception ex) {
       throw new InboxListenerException("Error inserting accounts (" +
                                         ex.getMessage() + ")");
     }
  }

  /**
   *
   */
  private void delete() throws InboxListenerException {
     try {
       System.out.print("username: ");
       String username = readValue();

       menuDAO.deleteAccount(username);

     } catch (Exception ex) {
       throw new InboxListenerException("Error deleting accounts (" +
                                         ex.getMessage() + ")");
     }
  }


  /**
   *
   */
  private int readInteger() throws InboxListenerException {

    InputStreamReader iReader = new InputStreamReader(System.in);
    BufferedReader indata     = new BufferedReader(iReader);

    int    value = -1;
    String temp  = null;
    try {
       temp = indata.readLine();
       value = Integer.parseInt(temp);
    } catch (IOException ex) {
       throw new InboxListenerException("Error reading input value (" +
                                         ex.getMessage() + ")");
    } catch (NumberFormatException ex) {
       throw new InboxListenerException("Error converting input value (" +
                                         ex.getMessage() + ")");
    }

    return value;
  }

  /**
   *
   */
  private String readValue() throws InboxListenerException {

    InputStreamReader iReader = new InputStreamReader(System.in);
    BufferedReader indata     = new BufferedReader(iReader);

    String value  = null;
    try {
       value = indata.readLine();
    } catch (IOException ex) {
       throw new InboxListenerException("Error reading input value (" +
                                         ex.getMessage() + ")");
    }

    return value;
  }

}
