/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.inboxlistener.engine;



import java.util.logging.Logger;
import java.util.logging.Level;

import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerConfigException;
import com.funambol.email.inboxlistener.admin.InboxListenerConfig;

/**
 *
 *
 * @version $Id: InboxListener.java,v 1.5 2006/05/31 12:14:23 gbmiglia Exp $
 */

public class InboxListener
{

  //------------------------------------------------------------------ CONSTANTS

  private static Logger log = Logger.getLogger(Def.LOGGER_NAME);

  //----------------------------------------------------------------- PROPERTIES


  //------------------------------------------------------------- PUBLIC METHODS

  /**
   * starting method
   *
   * @param args String[]
   */
  public static void main(String args[]) {

     // load the listener configuration
     String  listenerId          = "";
     boolean startupNotification = false;
     try {
        listenerId          = InboxListenerConfig.getConfigInstance().getListenerId();
        startupNotification = InboxListenerConfig.getConfigInstance().getStartupNotification();
     } catch (InboxListenerConfigException e) {
       e.printStackTrace();
     }

     // load Mail Server Properties
     String host        = "";
     String protocol    = "";
     boolean isSSL      = false;
     int    port        = 0;
     int    interval    = 0;
      String keystorePath = "";
     String keystorePassphrase = "";
     MSAccount msa      = null;
     try {
        host        = InboxListenerConfig.getConfigInstance().getHost();
        protocol    = InboxListenerConfig.getConfigInstance().getProtocol();
        isSSL       = InboxListenerConfig.getConfigInstance().getIsSSL();
        port        = InboxListenerConfig.getConfigInstance().getPort();
        interval    = InboxListenerConfig.getConfigInstance().getInterval();
        keystorePath = InboxListenerConfig.getConfigInstance().getKeystorePath();
        keystorePassphrase = InboxListenerConfig.getConfigInstance().getKeystorePassphrase();
        msa         = new MSAccount(host,
                                    isSSL,
                                    protocol,
                                    port,
                                    interval,
                                    keystorePath,
                                    keystorePassphrase);
     } catch (InboxListenerConfigException e) {
       e.printStackTrace();
     }

     // load DataBase Properties
     DBAccount dba        = null;
     String dbName        = "";
     String jdbcDriver    = "";
     String jdbcUrl       = "";
     String jdbcUser      = "";
     String jdbcPassword  = "";
     int    reloadingTime = 0;
     try {
        jdbcDriver    = InboxListenerConfig.getConfigInstance().getJdbcDriver();
        jdbcUrl       = InboxListenerConfig.getConfigInstance().getJdbcUrl();
        jdbcUser      = InboxListenerConfig.getConfigInstance().getJdbcUser();
        jdbcPassword  = InboxListenerConfig.getConfigInstance().getJdbcPassword();
        reloadingTime = InboxListenerConfig.getConfigInstance().getReloadingTime();
        dba          = new DBAccount(jdbcDriver,
                                     jdbcUrl,
                                     jdbcUser,
                                     jdbcPassword,
                                     reloadingTime);
     } catch (InboxListenerConfigException e) {
       e.printStackTrace();
     }

     // load WebServices Properties
     WSAccount wsa       = null;
     String wsEndPoint   = null;
     String wsUsername   = null;
     String wsPassword   = null;
     String wsSyncSource = null;
     try {
        wsEndPoint   = InboxListenerConfig.getConfigInstance().getWsEndPoint();
        wsUsername   = InboxListenerConfig.getConfigInstance().getWsUsername();
        wsPassword   = InboxListenerConfig.getConfigInstance().getWsPassword();
        wsSyncSource = InboxListenerConfig.getConfigInstance().getWsSyncSource();
        wsa          = new WSAccount(wsEndPoint,
                                     wsUsername,
                                     wsPassword,
                                     wsSyncSource);
     } catch (InboxListenerConfigException e) {
       e.printStackTrace();
     }

     if (log.isLoggable(Level.FINEST)) {
         log.finest("Start Inbox Listener "                      +
                    "\nInboxListener Parameters"                 +
                    "\nlistener id         : " + listenerId          +
                    "\nstartupNotification : " + startupNotification +
                    "\nMail Server Parameters"                   +
                    "\nHost         : " + msa.getHost()          +
                    "\nProtocol     : " + msa.getProtocol()      +
                    "\nSSL          : " + msa.getIsSSL()         +
                    "\nKeystorePath : " + msa.getKeystorePath()  +
                    "\nport         : " + msa.getPort()          +
                    "\ninterval     : " + msa.getInterval()      +
                    "\nDataBase Parameters"                      +
                    "\njdbcDriver   : " + dba.getJdbcDriver()    +
                    "\njdbcUrl      : " + dba.getJdbcUrl()       +
                    "\njdbcUser     : " + dba.getJdbcUser()      +
                    "\nreloadingTime: " + dba.getReloadingTime() +
                    "\nWebServices Parameters"                   +
                    "\nwsEndPoint   : " + wsa.getWsEndPoint()    +
                    "\nwsUsername   : " + wsa.getWsUsername()    +
                    "\nwsPassword   : " + wsa.getWsPassword()    +
                    "\nwsSyncSource : " + wsa.getWsSyncSource()  );
     }


     // start listener registry
     ListenerRegistry lr = new ListenerRegistry(listenerId,
                                                startupNotification,
                                                dba,
                                                msa,
                                                wsa);
     lr.setName("ListenerRegistry-"+listenerId);
     lr.start();

  }

}
