/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.items.dao;


import com.funambol.email.exception.EntityException;
import com.funambol.email.items.model.EmailFilter;
import com.funambol.email.items.model.ItemMessage;
import com.funambol.email.util.Utility;
import com.funambol.framework.tools.DBTools;
import com.funambol.server.tools.IdSpaceGenerator;
import com.sun.mail.pop3.POP3Folder;

import javax.mail.Flags;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.internet.MimeMessage;
import javax.mail.search.MessageIDTerm;
import javax.mail.search.SearchTerm;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.*;
import java.util.ArrayList;
import java.util.Locale;
import java.util.logging.Level;

/**
 * This class implements methods to access data
 * in Mail Server datastore.
 *
 * @version $Id: PopEntityDAO.java,v 1.18 2006/05/11 08:08:41 gbmiglia Exp $
 */
public class PopEntityDAO extends EntityDAO {

    // ------------------------------------------------------------ Private data

    // -------------------------------------------------------------- Properties

    /**
     *
     */
    public PopEntityDAO() {
    }

    /**
     * @param jndiDataSourceName String
     */
    public PopEntityDAO(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }

    // -----------------------------------------------------------Public Methods

    /**
     * @param FID         IMAPFolder ID
     * @param msg         Message
     * @param idSentSpace The GUID generator
     * @param source_uri  The SyncSource URI.
     * @param principalId The principal ID.
     * @return GUID String
     * @throws EntityException
     */
    public String addEmail(String FID,
                           Message msg,
                           IdSpaceGenerator idSentSpace,
                           String source_uri,
                           long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start addItemInLocalDB folder " + FID);
        }

        String GUID = null;
        Connection connection = null;
        PreparedStatement ps = null;

        try {
            lookupDataSource();
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(INSERT_SENT);

            GUID = Utility.getGUID(FID, idSentSpace.next());

            ps.setString(1, GUID);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            ps.setBytes(4, mail2db(msg));
            ps.executeUpdate();
            ps.close();
            ps = null;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }
        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO end insertItem UID " + GUID);
        }

        return GUID;
    }

    /**
     *
     * add one folder in the local DB
     *
     * @param fullPath folder full path
     * @param parentId parent folder id
     * @param idFolderSpace IdSpaceGenerator
     * @param source_uri String
     * @param principalId long
     * @return String
     * @throws EntityException
     */
    /*
    public String addFolder(String fullPath,
                            String parentId,
                            IdSpaceGenerator idFolderSpace,
                            String source_uri,
                            long principalId)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("DAO start addFolderInLocalDB folder ");
        }

        String GUID = null;
        try {
            GUID = insertFolder(fullPath, parentId, idFolderSpace,
                                source_uri, principalId);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        if (log.isLoggable(Level.FINEST)) {
          log.finest("DAO end insertFolder UID " + GUID );
        }

        return GUID;
    }
     */

    /**
     * gets all Messages
     *
     * @param f      POP3Folder
     * @param filter EmailFilter
     * @param loc Locale
     * @return Message[]
     * @throws EntityException
     */
    public Message[] getAllEmailsInbox(POP3Folder f, 
                                       EmailFilter filter, 
                                       Locale loc)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start getAllItems ");
        }

        Message[] messages;
        Message[] tmpmessages;
        ArrayList filteredmessages = new ArrayList();
        Message msg;

        try {

            if (filter.getId() != null) {
                String uid = Utility.getIdAfterSeparator(filter.getId());
                msg = getMessageByUID(f, uid);
                messages = new Message[1];
                messages[0] = msg;
            } else {
                if (filter.getTime() != null) {
                    // in the pop protocol this feature doesn't work
                    //SearchTerm st = new ReceivedDateTerm(ComparisonTerm.GE, filter.getTime());
                    //messages = f.search(st);
                    // manual filter
                    tmpmessages = f.getMessages();
                    java.util.Date d1   = null; 
                    //Date d2   = null; 
                    for (int i=0; i<tmpmessages.length; i++){
                        d1 = Utility.getHeaderDate(tmpmessages[i], loc); 
                        //d2 = tmpmessages[i].getReceivedDate();
                        if (d1.after(filter.getTime())){
                           filteredmessages.add(tmpmessages[i]);
                        }
                    }
                    messages = (Message[])filteredmessages.toArray(new Message[0]);
                } else {
                    messages = f.getMessages();
                }
            }

            if (messages != null) {
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("number of items " + messages.length);
                }
            }
        }
        catch (MessagingException me) {
            throw new EntityException(me);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO end getAllItems ");
        }

        return messages;

    }

    /**
     * @param filter
     * @param source_uri  the SyncSource URI.
     * @param principalId The principal ID.
     * @return The items fulfilling the filter requirements as an Array of Strings.
     * @throws EntityException
     */
    public String[] getAllEmailsSent(EmailFilter filter,
                                     String source_uri,
                                     long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start getAllItems ");
        }

        String[] ids = null;
        Connection connection = null;
        PreparedStatement ps = null;
        ResultSet rs;
        ArrayList rows;
        try {
            lookupDataSource();
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(SELECT_ALL_SENT);
            ps.setString(1, source_uri);
            ps.setLong(2, principalId);
            rows = new ArrayList();
            rs = ps.executeQuery();
            while (rs.next()) {
                rows.add(rs.getString(1));
            }
            ids = (String[]) rows.toArray(new String[0]);
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }
        if (ids != null) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("number of items " + ids.length);
            }
        }
        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO end getAllItems ");
        }

        return ids;

    }

    /**
     * @param f   IMAPFolder
     * @param uid long
     * @throws EntityException
     */
    public void removeEmail(POP3Folder f, String uid)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("POP DAO start removePopItem ");
        }

        try {
            Message msg = getMessageByUID(f, uid);
            if (msg != null) {
                msg.setFlag(Flags.Flag.DELETED, true);
            }
        } catch (MessagingException me) {
            throw new EntityException(me);
        }

    }

    /**
     * @param f POP3Folder
     * @throws EntityException
     */
    public void removeAllEmail(POP3Folder f)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("POP3 DAO start removeAllEmail ");
        }

        Message[] messages;
        try {

            messages = f.getMessages();

            for (int i = 0; i < messages.length; i++) {
                messages[i].setFlag(Flags.Flag.DELETED, true);
            }

        } catch (MessagingException e) {
            throw new EntityException(e);
        }

    }

    /**
     * serch a document using UID
     * for POP3  protocol UID is String
     *
     * @param f   POP3Folder
     * @param uid String
     * @return Message
     * @throws EntityException
     */
    public Message getEmailFromUID(POP3Folder f, String uid)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("POPDAO start getItemByUID folder: " + f + " uid: " + uid);
        }


        Message msg;
        try {
            msg = getMessageByUID(f, uid);
        } catch (MessagingException me) {
            throw new EntityException(me);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("POPDAO end getItemByUID ");
        }

        return msg;
    }

    /**
     * @param GUID        the Global Unique ID.
     * @param s           The session info to the backend.
     * @param source_uri  the SyncSource URI.
     * @param principalId The principal ID.
     * @return Message
     * @throws EntityException
     */
    public Message getEmailFromUID(String GUID,
                                   Session s,
                                   String source_uri,
                                   long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("POPDAO start getItemByUIDInSent GUID: " + GUID);
        }

        Message message = null;
        Connection connection = null;
        PreparedStatement ps = null;
        ResultSet rs;

        try {
            lookupDataSource();
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(SELECT_SENT);
            ps.setString(1, GUID);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            rs = ps.executeQuery();
            while (rs.next()) {
                message = db2mail(getBytesFromColumn(1, rs), s);
            }
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("POPDAO end getItemByUIDInSent ");
        }

        return message;

    }

    /**
     * @param GUID        the Global Unique ID.
     * @param source_uri  the SyncSource URI.
     * @param principalId The principal ID.
     * @throws EntityException
     */
    public void removeEmail(String GUID,
                            String source_uri,
                            long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("POP DAO start removeItemInSent UID " + GUID);
        }
        Connection connection = null;
        PreparedStatement ps = null;
        try {
            lookupDataSource();
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(DELETE_SENT);
            ps.setString(1, GUID);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            ps.executeUpdate();
            ps.close();
            ps = null;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("POP DAO end removeItemInSent UID " + GUID);
        }

    }


    /**
     * serch a document using the header Message-ID
     *
     * @param f          POP3Folder
     * @param parentId   String
     * @param message_id String
     * @return ItemMessage
     * @throws EntityException
     */
    public ItemMessage getEmailFromMessageID(POP3Folder f,
                                             String parentId,
                                             String message_id)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start getItemByMessageID Header Message-ID: " +
                    message_id);
        }

        ItemMessage eisMessage = null;
        Message msg;

        try {
            SearchTerm st = new MessageIDTerm(message_id);
            Message[] messages = f.search(st);

            if (messages != null) {

                if (log.isLoggable(Level.FINEST)) {
                    log.finest("Messages Found Message-ID: " + messages.length);
                }

                if (messages.length > 0) {

                    msg = messages[0];

                    String messid;

                    if (msg != null) {

                        messid = Utility.getHeaderMessageID(msg);

                        String FMID = f.getUID(msg);

                        if (message_id.equalsIgnoreCase(messid)) {

                            if (log.isLoggable(Level.FINEST)) {
                                log.finest("Email already Exist");
                            }

                            eisMessage = new ItemMessage(Utility.getGUID(parentId, FMID),
                                    parentId, messid, msg,
                                    null, null, null, null, null,
                                    null, null, null, null, null);
                        }
                    }
                }
            }
        } catch (MessagingException me) {
            throw new EntityException(me);
        }


        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO end getItemByMessageID message ");
        }

        return eisMessage;
    }

    /**
     * serch a document using the header Message-ID
     *
     * @param message_id  String
     * @param s           Session
     * @param source_uri  String
     * @param principalId long
     * @return ItemMessage
     * @throws EntityException
     */
    public ItemMessage getEmailFromMessageID(String message_id,
                                             Session s,
                                             String source_uri,
                                             long principalId)
            throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO start getItemByMessageIDInSent Header Message-ID: " +
                    message_id);
        }

        ItemMessage eisMessage = null;
        String GUID = null;
        Message javaMessage = null;
        Connection connection = null;
        PreparedStatement ps = null;
        ResultSet rs;

        try {
            lookupDataSource();
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(SELECT_TWIN_SENT);
            ps.setString(1, message_id);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            rs = ps.executeQuery();
            while (rs.next()) {
                GUID = rs.getString(1);
                javaMessage = db2mail(getBytesFromColumn(2, rs), s);
            }
            eisMessage = new ItemMessage(GUID, "S", message_id, javaMessage,
                    null, null, null, null, null, null,
                    null, null, null, null);
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DAO end getItemByMessageIDInSent Header Message-ID: " +
                    message_id);
        }

        return eisMessage;
    }

    //----------------------------------------------------------- PRIVATE METHODS

    /**
     * used by the SENT/OUTBOX synchronization
     *
     * @param msg Message
     * @return byte[]
     * @throws Exception
     */
    private static byte[] mail2db(Message msg) throws Exception {
        OutputStream baos = new ByteArrayOutputStream();
        msg.writeTo(baos);
        String msgS = baos.toString();
        return msgS.getBytes();
    }

    /**
     * used by the SENT/OUTBOX synchronization
     *
     * @param b byte[]
     * @param s Session
     * @return Message
     * @throws Exception
     */
    private Message db2mail(byte[] b, Session s) throws Exception {
        InputStream is = new ByteArrayInputStream(b);
        return new MimeMessage(s, is);
    }

    /**
     *
     * get the email from the blob column in the DB
     *
     * @param  colIndex
     * @param  rs
     * @return email from the DB byte[]
     * @throws SQLException
     */
    private byte[] getBytesFromColumn(int colIndex, ResultSet rs)
            throws SQLException {

        Object obj = rs.getObject(colIndex);
        if (obj instanceof Blob) {
            return ((Blob) obj).getBytes(1, (int) ((Blob) obj).length());
        }
        if (obj instanceof Clob) {
            return ((Clob) obj).getSubString(1, (int) ((Clob) obj).length()).getBytes();
        }
        if (obj instanceof byte[]) {
            return (byte[]) obj;
        }
        if (obj instanceof String) {
            return ((String) obj).getBytes();
        }
        throw new ClassCastException("Unexpected type " + obj.getClass().getName() + ".");
    }
    
    
    /**
     * workaround.
     * In the POP folder there is not the method
     * getMessageByUID
     *
     * @param f POP3Folder
     * @param uid String
     * @return Message
     * @throws MessagingException
     */
    private Message getMessageByUID(POP3Folder f, String uid)
       throws MessagingException{
        Message msg = null;

        Message[] messages = f.getMessages();

        int n = messages.length;
        String uidTmp = null;

        for (int i=0; i<n; i++) {
            msg = messages[i];
            uidTmp = f.getUID(msg);

            if (uidTmp != null){
                if (uidTmp.equalsIgnoreCase(uid)) {
                    break;
                }
            }
        }

        return msg;
  }    

}