/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.items.manager ;


import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Locale;
import java.io.InputStream;
import java.io.ByteArrayInputStream;

import javax.mail.Message;
import javax.mail.Session;
import javax.mail.Folder;
import javax.mail.Flags;
import javax.mail.Transport;
import com.sun.mail.imap.IMAPFolder;
import javax.mail.internet.MimeMessage;
import javax.mail.MessagingException;

import com.funambol.email.pdi.mail.Email;
import com.funambol.email.pdi.mail.EmailItem;
import com.funambol.email.items.dao.EntityDAO;
import com.funambol.email.items.model.ItemMessage;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.CommonMailServerWrapper;
import com.funambol.email.transport.ImapMailServerWrapper;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.exception.*;
import com.funambol.email.items.model.ItemFolder;
import com.funambol.email.items.model.DefaultFolder;
import com.funambol.email.items.model.EmailFilter;
import com.funambol.server.tools.IdSpaceGenerator;
import java.util.Date;
import javax.mail.Address;
import javax.mail.SendFailedException;




/**
 *
 * @version $Id: EntityManager.java,v 1.30 2006/07/02 10:19:19 gbmiglia Exp $
 */
public class EntityManager {



    // -------------------------------------------------------------- Properties

    /**     */
    protected CommonMailServerWrapper cmsw = null;

    /**     */
    protected Logger  log = Logger.getLogger(Def.LOGGER_NAME);

    /**     */
    public EntityDAO ed       = null;

    /**     */
    protected long timeStart = 0;
    protected long timeStop  = 0;

    // ------------------------------------------------------------ Costructors


    /**
     *
     */
    public EntityManager(){
    }

    /**
     *
     *
     * @param msw_e IMailServerWrapper
     * @param jndiDataSourceName String
     * @throws EmailAccessException
     */
    public EntityManager(IMailServerWrapper msw_e, String jndiDataSourceName)
     throws EmailAccessException
    {
       cmsw = (CommonMailServerWrapper)msw_e;
       ed   = new EntityDAO(jndiDataSourceName);
    }

   // ----------------------------------------------------------- Public Methods
   
    /**
     *
     * create a javaMail Message from a funambol foundation Email
     *
     * @param s Session
     * @param ctmp Email
     * @param FID String
     * @param toBeSend boolean
     * @return Message
     * @throws EntityException
     */
    public Message createMessage(Session s,
                                 Email ctmp,
                                 String FID,
                                 boolean toBeSend,
                                 boolean read,
                                 boolean forwarded,
                                 boolean replied)
      throws EntityException {

        Message doc;
        
        try {

            EmailItem emailItem = ctmp.getEmailItem();

            String rfc2822 = emailItem.getStringRFC2822();
            
            String messageID = Utility.getHeaderMessageID(rfc2822);
            
            byte[] b = rfc2822.getBytes();
            
            InputStream is = new ByteArrayInputStream(b);

            doc = new MimeMessage(s, is);

            // if draft set the FLAG
            if (FID.equalsIgnoreCase(Def.FOLDER_DRAFTS_ID)){
               doc.setFlag(Flags.Flag.DRAFT, true);
            }
                        
            // for all folders set eventually the Flags
            doc.setFlags(new Flags(Flags.Flag.SEEN)     , read);
            //doc.setFlags(new Flags(Flags.Flag.FORWARDED), false);
            doc.setFlags(new Flags(Flags.Flag.ANSWERED) , replied);
            
            if (toBeSend){
               // don't set the HEADER_LABEL_SYNCMESSAGE
            } else {
               // we must set the message as "sync message"
               doc.setHeader(Def.HEADER_LABEL_SYNCMESSAGE, Def.HEADER_VALUE_SYNCMESSAGE);
            }
            
            if (messageID != null){
               doc.setHeader(Def.HEADER_LABEL_MESSAGE_ID,messageID);
            }

        } catch (Exception e){
            throw new EntityException(e);
        }
        
        return doc;
    }             
    

    /**
     *
     *
     * @param smsg ItemMessage
     * @return Email
     * @throws EntityException
     */
    public Email createFoundationMail(ItemMessage smsg)
     throws EntityException {

        Email mtmp       = new Email();

        String guid      = null;  // parentId + separator + mailId
        String parentId  = null;

        String emailItem = null;
        String encodingType = null;

        String created   = null; // Date created
        String received  = null; // Date received
        String modified  = null; // Date modified
        String read      = null; // boolean read
        String forwarded = null; // boolean forwarded
        String replied   = null; // boolean replied
        String deleted   = null; // boolean deleted
        String flagged   = null; // boolean flagged

        try {

           guid      = smsg.getGUID();
           parentId  = smsg.getParentId();

           emailItem = smsg.getStreamMessage();
           encodingType = smsg.getEncodingType();

           created   = smsg.getCreated();
           received  = smsg.getReceived();
           modified  = smsg.getModified();
           read      = smsg.getRead();
           forwarded = smsg.getForwarded();
           replied   = smsg.getReplied();
           deleted   = smsg.getDeleted();
           flagged   = smsg.getFlagged();

        } catch (Exception e) {
           e.printStackTrace();
        }

        mtmp.getUID().setPropertyValue(guid);
        
        mtmp.getParentId().setPropertyValue(parentId);

        mtmp.getEmailItem().setPropertyValue(emailItem);
        mtmp.getEmailItem().setEncoding(encodingType);

        if (created != null){
          mtmp.getCreated().setPropertyValue(created);
        }
        if (received != null){
          mtmp.getReceived().setPropertyValue(received);
        }
        if (modified != null){
          mtmp.getModified().setPropertyValue(modified);
        }

        if (read != null){
          mtmp.getRead().setPropertyValue(read);
        }
        if (forwarded != null){
          mtmp.getForwarded().setPropertyValue(forwarded);
        }
        if (replied != null){
          mtmp.getReplied().setPropertyValue(replied);
        }
        if (deleted != null){
          mtmp.getDeleted().setPropertyValue(deleted);
        }
        if (flagged != null){
          mtmp.getFlagged().setPropertyValue(flagged);
        }

        return mtmp;
    } 
    
    
    /**
     *
     * create the foundation folder object 
     *
     * @param f ItemFolder
     * @return Folder
     * @throws EntityException
     */
    public com.funambol.email.pdi.folder.Folder createFoundationFolder(
                                                               ItemFolder f)
      throws EntityException {

        com.funambol.email.pdi.folder.Folder fout = new 
                                         com.funambol.email.pdi.folder.Folder();

        fout.getUID().setPropertyValue(f.getGUID());

        fout.getParentId().setPropertyValue(f.getParentId());

        fout.getName().setPropertyValue(f.getName());

        fout.getCreated().setPropertyValue(f.getCreated());

        fout.getAccessed().setPropertyValue(f.getAccessed());

        fout.getModified().setPropertyValue(f.getModified());

        fout.getRole().setPropertyValue(f.getRole());

        //fout.getAttributes().setPropertyValue(f.getAttributes);

        return fout;
    }

    
    /**
     *
     * insert the default folder if needed
     *
     *
     * @param df DefaultFolder
     * @param source_uri String
     * @param pID principal ID long
     * @throws EntityException
     */
    public void  insertDefaultFolder(DefaultFolder df,
                                     String protocol,
                                     String source_uri,
                                     long pID)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertDefaultFolder start");
        }

        String GUID       = null;
        String folderName = null;
        String parentId   = Def.FOLDER_ROOT_ID ;

        try {

          GUID       = Utility.getGUID(parentId, df.getInboxId());
          folderName = df.getInboxName();
          addDefaultFolder(df, "I", GUID, folderName, parentId, protocol,
                           source_uri, pID);
          
          GUID       = Utility.getGUID(Def.FOLDER_ROOT_ID, df.getOutboxId());
          folderName = df.getOutboxName();
          addDefaultFolder(df, "O", GUID, folderName, parentId, protocol,
                           source_uri, pID);
          
          GUID       = Utility.getGUID(Def.FOLDER_ROOT_ID, df.getSentId());
          folderName = df.getSentName();
          addDefaultFolder(df, "S", GUID, folderName, parentId, protocol,
                           source_uri, pID);
                    
          GUID       = Utility.getGUID(Def.FOLDER_ROOT_ID, df.getDraftId());
          folderName = df.getDraftName();
          addDefaultFolder(df, "D", GUID, folderName, parentId, protocol,
                           source_uri, pID);

          GUID       = Utility.getGUID(Def.FOLDER_ROOT_ID, df.getTrashId());
          folderName = df.getTrashName();
          addDefaultFolder(df, "T", GUID, folderName, parentId, protocol,
                           source_uri, pID);

          
       } catch (EntityException ee){
           throw new EntityException("Error inserting the default folder", ee);
       }

       if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertDefaultFolder end" );
       }

    }
    
    
   /**
     *
     * Check if the local DB folders are consistent with the MailBox folders
     *
     *
     * @param source_uri String
     * @param principalId long
     * @throws EntityException
     */
    public void  insertCustomFolder(IMailServerWrapper msw,
                                    boolean allMailboxActivation,
                                    IdSpaceGenerator idFolderSpace,
                                    String source_uri, 
                                    long principalId)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertCustomFolder start");
        }

        try {
            
            if (msw instanceof ImapMailServerWrapper){
                           
               ImapMailServerWrapper imsw = (ImapMailServerWrapper)msw;

                if (allMailboxActivation){
                    String     defFullpath;
                    IMAPFolder imapf;

                    // insert all folders in Inboxbox + inboxbox and so on
                    defFullpath = imsw.getDefaultFolder().getInboxName();
                    imapf       = imsw.getInboxFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                    defFullpath = imsw.getDefaultFolder().getOutboxName();
                    imapf       = imsw.getOutboxFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                    defFullpath = imsw.getDefaultFolder().getSentName();
                    imapf       = imsw.getSentFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                    defFullpath = imsw.getDefaultFolder().getDraftName();
                    imapf       = imsw.getDraftsFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                    defFullpath = imsw.getDefaultFolder().getTrashName();
                    imapf       = imsw.getTrashFolder();
                    this.ed.insertCustomFolder(imapf, defFullpath, idFolderSpace,
                                               source_uri, principalId);

                }
                              
            }
               
        } catch (EmailAccessException ee){
           throw new EntityException("Error getting all folder", ee);
        } catch (EntityException ee){
           throw new EntityException("Error inserting the custom folder", ee);
        }

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertCustomFolder end" );
        }

    }
    
    
    /**
     *
     * Check if the item is a folder or an email
     * using the SyncItemKey
     *
     *
     * @param parentId String
     * @param objectId String
     * @param source_uri String
     * @param principalId long
     * @return boolean
     * @throws EntityException
     */
    public boolean isEmail(String parentId,
                           String objectId,
                           String source_uri,
                           long principalId)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager isEmail start");
        }

        boolean ismail = false;
        ItemFolder it;


        try {

           it = this.ed.getFolderFromUID(Utility.getGUID(parentId, objectId), 
                                         source_uri, 
                                         principalId);

           if (it == null){
              
              ismail = true;
               
              /* 
              // try to get a mail
              String parentGUID = it.getParentId();
              String parentfullpath = this.ed.getFullPathFromGUID(parentGUID,
                                                                  source_uri,
                                                                  principalId);              
              Folder f = 
                     this.cmsw.getMailDefaultFolder().getFolder(parentfullpath);
              
              try {
                  long uid = Long.parseLong(objectId);
                  // if the parse can convert the id we have to go
                  // on a IMAPFolder
                  IMAPFolder imapFolder = (IMAPFolder)f;
                  Message m = imapFolder.getMessageByUID(uid);
                  if (m != null){
                     ismail = true;
                  }
              } catch (NumberFormatException  nfe){
                  // if the parse can't convert the id we have to go
                  // on a POP3Folder
                  POP3Folder pop3Folder = (POP3Folder)f;
                  Message m = this.ed.getMessageByUID(pop3Folder, objectId);
                  if (m != null){
                     ismail = true;
                  }
              }
              */
           }

        } catch (EntityException ee){
           throw new EntityException("Error determinig object type (mail/folder)",
                                     ee);
        }

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager isEmail end (isEmail :" + ismail + ")" );
        }

        return ismail;

    }

    // -------------------------------------------------------- Protected Methods

     /**
      * convert Message into ServerMessages
      *
      *
      * @param GUID unique folder id
      * @param name folder name
      * @param FID parent id
      * @return ItemFolder
      * @throws EntityException
      */
     public ItemFolder setItemFolder(String GUID,
                                        String name,
                                        String FID,
                                        String folderRole,
                                        String creationDate)
       throws EntityException {

         if (log.isLoggable(Level.FINEST)) {
             log.finest("setItemFolder() start " + GUID);
         }

         ItemFolder    clientFoler;

         try {



             clientFoler = new ItemFolder(GUID,
                                         name,
                                         FID,
                                         creationDate,
                                         null, // accessed
                                         null, // modified
                                         folderRole,
                                         null // attributes
                                         );

         } catch (Exception e){
             throw new EntityException(e);
         }

         if (log.isLoggable(Level.FINEST)) {
             log.finest("setItemFolder() - end ");
         }

         return clientFoler;

     }

     /**
      *
      * convert Message into ServerMessages
      *
      * @param GUID unique mail id
      * @param msg Message
      * @param parentId String
      * @param loc Locale
      * @param encodingType String
      * @return ItemMessage
      * @throws EntityException
      */
     public ItemMessage setItemMessage(String GUID,
                                       String parentId,
                                       Message msg,
                                       boolean _read,
                                       boolean _forwarded, 
                                       boolean _replied,
                                       Locale loc,
                                       String encodingType)
       throws EntityException {

         ItemMessage   clientMsg;
         String        message_id;
         String        stream;
         String        created;
         String        received;
         String        modified;
         String        read;
         String        forwarded;
         String        replied;
         String        deleted;
         String        flagged;

         try {

             // header message-id
             message_id = Utility.getHeaderMessageID(msg);

             // message
             stream = Utility.getStreamFromMessage(msg, encodingType);

             //origination date
             created = Utility.UtcFromDate(Utility.getHeaderDate(msg, loc));
             //received date
             received = Utility.getReceivedDate(msg, loc);
             //modified date
             modified = Utility.UtcFromDate(Utility.getHeaderDate(msg, loc)); // @todo: tmp

             // flags
             read      = String.valueOf(_read);
             forwarded = String.valueOf(_forwarded);
             replied   = String.valueOf(_replied);
             deleted   = String.valueOf(msg.isSet(Flags.Flag.DELETED));
             flagged   = null; //@todo: tmp String.valueOf(msg.isSet(Flags.Flag.FLAGGED));

             clientMsg = new ItemMessage(GUID,
                                         parentId,
                                         message_id,
                                         msg,
                                         stream,
                                         encodingType,
                                         created,
                                         received,
                                         modified,
                                         read,
                                         forwarded,
                                         replied,
                                         deleted,
                                         flagged);

         } catch (MessagingException e){
             throw new EntityException(e);
         }

         return clientMsg;

     }
     
  //---------------------------------------------------------- PROTECTED METHODS
     
   /**
    * sends the email
    *
    * @param msg javaMail message to send; Message 
    * @throws EntityException
    */
    protected void sendEmail(Message msg) throws SendingException {
        
        String subject = null;
        
        try {

            subject = msg.getSubject();
            
            timeStart = System.currentTimeMillis();
               
            Transport.send(msg);
               
            timeStop = System.currentTimeMillis();
            
            if (log.isLoggable(Level.FINEST)) {
               log.finest("Sending Execution Time: " + (timeStop - timeStart) + " ms");
               
            }
        
        } catch (MessagingException me) {
            
            String errorMessage = "Error sending email " + subject;
            
            String invalidList     = "";
            String validUnsentList = "";
            String validSentList   = "";
            
            Exception ex = me;
            
            do {
                if (ex instanceof SendFailedException) {
                    
                    SendFailedException sfex = (SendFailedException) ex;
                    
                    Address[] invalid = sfex.getInvalidAddresses();
                    if (invalid != null) {
                        for (int i = 0; i < invalid.length; i++) {
                            invalidList = invalidList + " " + invalid[i];
                        }
                        if (!invalidList.trim().equals("")){
                           invalidList = " Invalid address: " + invalidList;
                        }
                    }
                    
                    Address[] validUnsent = sfex.getValidUnsentAddresses();
                    if (validUnsent != null) {
                        for (int i = 0; i < validUnsent.length; i++) {
                            validUnsentList = validUnsentList + " " + validUnsent[i];
                        } 
                        if (!validUnsentList.trim().equals("")){
                           validUnsentList = " Valid unsent address: " + validUnsentList;
                        }
                    }
                    
                    Address[] validSent = sfex.getValidSentAddresses();
                    if (validSent != null) {
                        for (int i = 0; i < validSent.length; i++) {
                            validSentList = validSentList + " " + validSent[i];
                        }
                        if (!validSentList.trim().equals("")){
                           validSentList = " Valid sent address: " + validSentList;
                        }
                    }
                    
                }

                if (ex instanceof MessagingException) {
                    ex = ((MessagingException) ex).getNextException();
                } else {
                    ex = null;
                }
                
            } while (ex != null);
            
            errorMessage = errorMessage + invalidList + validUnsentList + validSentList;
            log.severe(errorMessage);            
            throw new SendingException(errorMessage);
            
        }
        
    }      
    
    
    /**
     *
     * @param msg Message
     * @param filter EmailFilter
     * @return true if message date match the filter date
     */
    protected boolean hasMatchedDate(Date msgdate, EmailFilter filter){
        
        boolean check = false;
                
        Date filterDate = filter.getTime();
        
        String timeFilterClause = filter.getTimeClause();
        
        if (timeFilterClause.equalsIgnoreCase("EQ") ||
            timeFilterClause.equalsIgnoreCase("GT") ||
            timeFilterClause.equalsIgnoreCase("GE")) {
            
            if (msgdate.after(filterDate)){
                check = true;
            }
            
        } else if (timeFilterClause.equalsIgnoreCase("LT") ||
                   timeFilterClause.equalsIgnoreCase("LE")) {

            if (msgdate.before(filterDate)){
                check = true;
            }
            
        } 
        
        return check;
    }
    
    
   /**
     *
     * @param msg Message
     * @param filter EmailFilter
     * @return true if message date match the filter date
     */
    protected boolean hasMatchedSize(Message msg, EmailFilter filter){
        
        boolean check = false;
        
        // if get all message return true
        if (filter.getSize() == Def.FILTER_SIZE_H_B_A){
            return true;
        } else {
            /**
             * @todo
             */        
            
        }
        
        // tmp
        check = true;
        
        return check;
    }
        
     
   /**
    * returns an array with three flags: read, forwarded, replied
    *
    * @param email EmailFID String
    * @return array with the read, forwarded, replied  flags
    *
    */ 
   protected boolean[] setFlags(Email email){
        
        String  flagS     = "false";
        
        boolean read      = false;
        boolean forwarded = false;
        boolean replied   = false;
        
        if (email != null) {
            flagS = (String)email.getRead().getPropertyValue();
            if (flagS != null){
               read = (Boolean.valueOf(flagS)).booleanValue();
            }
            flagS = (String)email.getForwarded().getPropertyValue();
            if (flagS != null){
               forwarded = (Boolean.valueOf(flagS)).booleanValue();
            }
            flagS = (String)email.getReplied().getPropertyValue();
            if (flagS != null){
               replied = (Boolean.valueOf(flagS)).booleanValue();
            }
        }
        
        boolean[] flags = { read, forwarded, replied };
        
        return flags;
       
   } 
    
  //------------------------------------------------------------ PRIVATE METHODS
     
    /**
     *
     * insert the default folder if needed
     *
     *
     * @param df DefaultFolder
     * @param source_uri String
     * @param principalId long
     * @throws EntityException
     */
    private void  addDefaultFolder(DefaultFolder df,
                                   String flag,
                                   String GUID,
                                   String folderName,
                                   String parentId, 
                                   String protocol,
                                   String source_uri,
                                   long pID)
      throws EntityException {

        if (log.isLoggable(Level.FINEST)) {
          log.finest("EntityManager insertDefaultFolder start");
        }

        String folderInDB = null;
        Folder folderInMS = null;
        Folder folderToInsert = null;

        try {

          if (protocol.equals(Def.PROTOCOL_IMAP)){  
             // IMAP PROTOCOL           
             folderInMS = this.ed.getDefaultFolder(this.cmsw, folderName);
             folderInDB = this.ed.getFullPathFromGUID(GUID, source_uri, pID);
             if (folderInMS == null){   
                 // is not in the mail server
                 if (folderInDB == null) {  
                    // it is not in the DB
                    this.ed.insertDefaultFolderInMailServer(this.cmsw, folderName);
                    this.ed.insertDefaultFolderInDB(GUID, parentId, folderName,
                                                    source_uri, pID);
                    setDefaultFolderPresence(df, flag, protocol); 
                 } else{  
                    // ... but is in the DB --> error
                 }
             } else {   
                 // is in the mail server
                 if (folderInDB != null){
                    // is in the DB --> do nothing
                    setDefaultFolderPresence(df, flag, protocol); 
                 } else {
                    // is not in the DB --> insert in the DB
                    this.ed.insertDefaultFolderInDB(GUID, parentId,  folderName,
                                                    source_uri, pID);
                    setDefaultFolderPresence(df, flag, protocol); 
                 }
             }
             
          } else {
             // POP3 PROTOCOL                 
             // folderInMS: for pop the only folder is the INBOX
             folderInDB = this.ed.getFullPathFromGUID(GUID, source_uri, pID);
             if (folderInDB == null) {  
                // it is not in the DB --> insert in the DB
                this.ed.insertDefaultFolderInDB(GUID, parentId,  folderName,
                                                source_uri, pID);
                setDefaultFolderPresence(df, flag, protocol); 
             }  
          }

                   
       } catch (EntityException ee){
           throw new EntityException("Error inserting the default folder", ee);
       }

    }   
    
    
    /**
     *
     * insert the default folder if needed
     *
     *
     * @param df DefaultFolder
     * @param source_uri String
     * @param principalId long
     * @throws EntityException
     */
    private void  setDefaultFolderPresence(DefaultFolder df,
                                           String flag,
                                           String protocol)
      throws EntityException {


        if        (flag.equals("I")){
            if (protocol.equals(Def.PROTOCOL_IMAP)){
                df.setIsInboxInMailServer(true);                      
            } else {
                df.setIsInboxInMailServer(true);                      
            }
        } else if (flag.equals("O")){
            if (protocol.equals(Def.PROTOCOL_IMAP)){
                df.setIsOutboxInMailServer(true);                      
            } else {
                df.setIsOutboxInMailServer(true);                      
            }
        } else if (flag.equals("S")){
            if (protocol.equals(Def.PROTOCOL_IMAP)){
                df.setIsSentInMailServer(true);                      
            } else {
                df.setIsSentInMailServer(true);                      
            }
        } else if (flag.equals("D")){
            if (protocol.equals(Def.PROTOCOL_IMAP)){
                df.setIsDraftInMailServer(true);                      
            } else {
                df.setIsDraftInMailServer(false);                      
            }
        } else if (flag.equals("T")){            
            if (protocol.equals(Def.PROTOCOL_IMAP)){
                df.setIsTrashInMailServer(true);                      
            } else {
                df.setIsTrashInMailServer(false);                      
            }
        }
        
    
    }      
    
}
