/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.items.manager;


import com.funambol.email.exception.EntityException;
import com.funambol.email.items.model.ItemMessage;
import com.funambol.email.items.model.InternalPart;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.framework.engine.SyncItemState;

import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.DataHandler;
import java.io.IOException;
import java.util.*;
import java.util.logging.Level;


/**
 * <p>Filter class. </p>
 */

public class EntityManagerFilter extends EntityManager {
    
    //---------------------------------------------------------------- CONSTANTS
    


    //----------------------------------------------------------- PUBLIC METHODS
    
    /**
     * returns only header
     *
     * @param session opened mail server session
     * @param FID     parent folder id
     * @param FMID    mail id in the folder
     * @param msg     Message
     * @param status  status of the item; char
     * @param loc     Locale
     * @return ItemMessage
     * @throws EntityException
     */
    public ItemMessage get_H(Session session,
                             String FID,
                             String FMID,
                             Message msg,
                             char status,
                             Locale loc)

      throws EntityException {


        if (log.isLoggable(Level.FINEST)) {
            log.finest(" start getPart_H ");
        }

        ItemMessage im = null;

        try {
            
            // save the flag
            boolean read      = msg.isSet(Flags.Flag.SEEN);
            boolean forwarded = false;
            boolean replied   = msg.isSet(Flags.Flag.ANSWERED);

            Message newMsg = new MimeMessage(session);

            // header
            String[] h = Utility.setNewHeader(newMsg, msg);
            String mid              = h[0];
            int    headersDimension = Integer.parseInt(h[1]);            

            // default empty content
            newMsg.setContent(Def.DEFAULT_CONTENT_VALUE_BODY, 
                              Def.DEFAULT_CONTENT_VALUE_CONTENTTYPE);
                        
            newMsg.saveChanges();
            
            // the save message command change the message-ID
            newMsg.setHeader(Def.HEADER_LABEL_MESSAGE_ID, mid);

            String encodingType = getEncodingType(newMsg);

            im = setItemMessage(FMID, FID, newMsg, 
                                read, forwarded, replied, 
                                loc, encodingType);

        } catch (MessagingException me) {
            throw new EntityException(me);
        }
        
        // remove the message if needed (syncml dimension optimization)
        if (Utility.needsFlagOptimization(status, FID)) {
            im.setStreamMessage("");
        }    

        return im;

    }


    /**
     *
     * returns header and body with byte quatity limitation
     *
     *
     * @param session  opened mail server session
     * @param FID      parent folder id
     * @param FMID     mail id in the folder
     * @param msg      Message
     * @param status   status of the item; char
     * @param loc      Locale
     * @param numBytes int
     * @return ItemMessage
     * @throws EntityException
     */
    public ItemMessage get_HB(Session session,
                              String FID,
                              String FMID,
                              Message msg,
                              char status,
                              Locale loc,
                              int numBytes)
      throws EntityException {

        ItemMessage im = null;

        
        try {

            // save the flag
            boolean read      = msg.isSet(Flags.Flag.SEEN);
            boolean forwarded = false;
            boolean replied   = msg.isSet(Flags.Flag.ANSWERED);
            
            Message newMsg = new MimeMessage(session);

            // header
            String[] h = Utility.setNewHeader(newMsg, msg);
            String mid              = h[0];
            int    headersDimension = Integer.parseInt(h[1]);
                
            // content
            if (Utility.needsFlagOptimization(status, FID)){
                // set empty content (will be removed)
                newMsg.setContent(Def.DEFAULT_CONTENT_VALUE_BODY, 
                                  Def.DEFAULT_CONTENT_VALUE_CONTENTTYPE);            
            } else{   
                // set the content (will not be removed)
                Object    content    = msg.getDataHandler().getContent();
                ArrayList newContent = new ArrayList();

                if (numBytes == -1){
                    // no filter for H+B
                    if (content instanceof Multipart) {
                        newContent = getParts((Multipart) content, -1, true);
                    } else {
                        getPartFromSingle(newContent, msg, -1, true);
                    }
                    InternalPart v = (InternalPart)newContent.get(0);
                    newMsg.setContent(v.getPartValue(), v.getContentType());
                    
                } else {
                    // body will be filtered
                    if (headersDimension < numBytes){

                        // remove from numbytes the header dimension
                        numBytes = numBytes - headersDimension;
                        
                        if (content instanceof Multipart) {
                           newContent = getParts((Multipart) content, numBytes, true);
                        } else {
                           getPartFromSingle(newContent, msg, numBytes, true);
                        }   
                        InternalPart v = (InternalPart)newContent.get(0);
                        newMsg.setContent(v.getPartValue(), v.getContentType());
                    } else {
                        // set empty content because the header is too large
                        newMsg.setContent(Def.DEFAULT_CONTENT_VALUE_BODY, 
                                          Def.DEFAULT_CONTENT_VALUE_CONTENTTYPE);            
                    }
                }
            }    
                   
            // re-set the flag on the server
            msg.setFlag(Flags.Flag.SEEN, read);
            //msg.setFlag(Flags.Flag.SEEN, forwarded);
            msg.setFlag(Flags.Flag.ANSWERED,replied);           
            
            newMsg.saveChanges();

            // the save message command change the message-ID
            newMsg.setHeader(Def.HEADER_LABEL_MESSAGE_ID, mid);
            
            String encodingType = getEncodingType(newMsg);

            im = setItemMessage(FMID, FID, newMsg,  
                                read, forwarded, replied, 
                                loc, encodingType);

        } catch (MessagingException me) {
            throw new EntityException(me);
        } catch (IOException ioe) {
            throw new EntityException(ioe);
        }
        
        // remove the message if needed (syncml dimension optimization)
        if (Utility.needsFlagOptimization(status, FID)){
            im.setStreamMessage("");
        }    
                
        return im;

    }



    /**
     *
     * if numBytes = -1 it returns all parts of the message 
     * (header, body, attachements) without byte quatity limitation.
     * otherwise it returns all parts of the message 
     * (header, body, attachements) but there is the byte number limitation
     *
     * @param session  opened mail server session
     * @param FID      parent folder id
     * @param FMID     mail id in the folder
     * @param msg      Message
     * @param status   status of the item; char
     * @param loc      Locale
     * @param numBytes int
     * @return ItemMessage
     * @throws EntityException
     */
    public ItemMessage get_HBA(Session session,
                               String FID,
                               String FMID,
                               Message msg,
                               char status,
                               Locale loc,
                               int numBytes)
      throws EntityException {

        ItemMessage im = null;
            
        try {

            // save the flag
            boolean read      = msg.isSet(Flags.Flag.SEEN);
            boolean forwarded = false;
            boolean replied   = msg.isSet(Flags.Flag.ANSWERED);
                        
            int messageSize   = msg.getSize();
            
            boolean onlyflags = Utility.needsFlagOptimization(status, FID);
               
            if (onlyflags){
                
                // optimization (get only flags)  
                Message newMsg = new MimeMessage(session);

                // header
                String[] h = Utility.setNewHeader(newMsg, msg);
                String mid              = h[0];
                int    headersDimension = Integer.parseInt(h[1]);                           
                
                // set empty content (will be removed)                
                newMsg.setContent(Def.DEFAULT_CONTENT_VALUE_BODY, 
                                  Def.DEFAULT_CONTENT_VALUE_CONTENTTYPE);
                
                newMsg.saveChanges();
                
                // the save message command change the message-ID
                newMsg.setHeader(Def.HEADER_LABEL_MESSAGE_ID, mid);                
                                
                String encodingType = getEncodingType(newMsg);

                im = setItemMessage(FMID, FID, newMsg, 
                                    read, forwarded, replied,
                                    loc, encodingType);                
            } else {
                
                if (numBytes == -1){
                    
                    // case get_HBA: get entire message without num bytes limitation 
                    String encodingType = getEncodingType(msg);
                    
                    im = setItemMessage(FMID, FID, msg,
                                        read, forwarded, replied,
                                        loc, encodingType); 
                    
                } else if (messageSize <= numBytes){
                    
                    // optimization (get entire message because the mail 
                    // is smaller than numBytes
                    String encodingType = getEncodingType(msg);
                    
                    im = setItemMessage(FMID, FID, msg,
                                        read, forwarded, replied,
                                        loc, encodingType);                                   
                } else {
                    
                    // filter the message
                    // case get_HBA: get message with num byte limitation
                    Message newMsg = new MimeMessage(session);

                    // header
                    String[] h = Utility.setNewHeader(newMsg, msg);
                    String mid              = h[0];
                    int    headersDimension = Integer.parseInt(h[1]);                           
                    
                    if (headersDimension < numBytes){
                        
                        // remove from numbytes the header dimension
                        numBytes = numBytes - headersDimension;
                        
                        // set filtered content (will not be removed)
                        Object content       = msg.getDataHandler().getContent();
                        ArrayList newContent = new ArrayList();

                        if (content instanceof Multipart) {
                            // get the old content in according with the filter
                            newContent = getParts((Multipart) content, numBytes, false);
                            // set the new content in the new email
                            if ( newContent.size() == 1){
                                BodyPart part = new MimeBodyPart();
                                InternalPart v = (InternalPart)newContent.get(0);
                                newMsg.setContent(v.getPartValue(), v.getContentType());
                            } else if (newContent.size() > 1){
                                BodyPart part;
                                MimeMultipart multipart = new MimeMultipart();
                                for (int i = 0; i < newContent.size(); i++) {
                                    InternalPart v = (InternalPart)newContent.get(i);
                                    part = new MimeBodyPart();
                                    part.setContent(v.getPartValue(), v.getContentType());
                                    if (newContent.size()>1){
                                       part.setDataHandler(v.getDataHandler());
                                    }    
                                    if (v.getDisposition() != null){
                                       part.setDisposition(v.getDisposition());
                                    }
                                    if (v.getFileName() != null){
                                       part.setFileName(v.getFileName());
                                    }
                                    multipart.addBodyPart(part);
                                }
                                newMsg.setContent(multipart);
                            } else {
                                // just in case of error
                                BodyPart part = new MimeBodyPart();
                                newMsg.setContent(Def.DEFAULT_CONTENT_VALUE_BODY, 
                                                  Def.DEFAULT_CONTENT_VALUE_CONTENTTYPE);
                            }
                        } else {
                            // get the old content in according with the filter
                            getPartFromSingle(newContent, msg, numBytes, false);
                            // set the new content
                            InternalPart v = (InternalPart)newContent.get(0);
                            newMsg.setContent(v.getPartValue(), v.getContentType());
                        }
                                        
                    } else {
                        // set empty content because the header is too large
                        newMsg.setContent(Def.DEFAULT_CONTENT_VALUE_BODY, 
                                          Def.DEFAULT_CONTENT_VALUE_CONTENTTYPE);            
                    } 
                    
                    // re-set the flag on the server
                    msg.setFlag(Flags.Flag.SEEN, read);
                    //msg.setFlag(Flags.Flag.SEEN, forwarded);
                    msg.setFlag(Flags.Flag.ANSWERED,replied);           
                                        
                    newMsg.saveChanges();

                    // the save message command change the message-ID
                    newMsg.setHeader(Def.HEADER_LABEL_MESSAGE_ID, mid);                

                    String encodingType = getEncodingType(newMsg);

                    im = setItemMessage(FMID, FID, newMsg, 
                                        read, forwarded, replied,
                                        loc, encodingType);                
                                        
                }   
                
            }
            
        } catch (MessagingException me) {
            throw new EntityException(me);
        } catch (IOException ioe) {
            throw new EntityException(ioe);
        }            
        
        // remove the message if needed (syncml dimension optimization)
        if (Utility.needsFlagOptimization(status, FID)){
            im.setStreamMessage("");
        }    

        return im;
    }

    /**
     *
     * check if message contains non-ASCII characters
     * and set the ccorrect encoding type
     *
     * @param msg Message 
     * @return encodingType String
     */
    public String getEncodingType(Message msg)  throws EntityException {
        
        String encodingType = null;
        boolean asciiOnly;
        
        try {
            asciiOnly = isASCIIOnly(msg);
        } catch (MessagingException e1) {
            throw new EntityException(e1);
        }

        if (asciiOnly){
            encodingType = null;
        } else {
            encodingType = Def.ENCODE_QUOTED_PRINTABLE;                
        }
        return encodingType;
    }    
    
    //---------------------------------------------------------- PRIVATE METHODS
    
    
    /**
     * get the Patrs of the email Content in according with 
     * the filter clause
     *
     * @param multipart Multipart
     * @param numBytes  int
     * @return InternalBody
     * @throws EntityException
     */
    private ArrayList getParts(Multipart multipart, 
                               int numBytes,
                               boolean getOnlyBody)
      throws EntityException {
        
        ArrayList    allParts          = new ArrayList();
        int          partsize          = 0;
        int          totalsize         = 0; 
        int          partNumber        = 0;
        Part         part              = null;
        InternalPart ip                = null;
        Object       mp                = null;    
        int          lastGottenPart    = 0;
        boolean      bodyAlreadyGotten = false;
        
        try {
            
            int partsCount = multipart.getCount();
                        
            for (int i = 0; i < partsCount; i++) {
                                
                part = multipart.getBodyPart(i);
                mp   = part.getContent();
        
                if (mp instanceof Multipart) {  
                    
                    // usually in this case we analize the body with 2 parts: 
                    // txt and html; we get only the txt part
                    
                    getPartFromMulti(allParts, (MimeMultipart) mp, numBytes, getOnlyBody);

                    // check the part to delete from list
                    for (int y = 0; y < allParts.size(); y++){
                        
                         ip = (InternalPart)allParts.get(y);
                         
                         if (getOnlyBody){                             
                             if (ip.isBody()){                                 
                                 if (ip.getContentType().equalsIgnoreCase("text/plain")){
                                    // stop after the body; exclude the attachment
                                    break; 
                                 } else{
                                    allParts.remove(y);
                                 }                             
                             } else {
                                 if (ip.getDisposition().equals(Part.INLINE) &&
                                     ip.getContentType().equalsIgnoreCase("text/plain")){
                                     // inline without "isBody" set
                                     break;
                                 }
                             }
                         } else {
                             if (ip.isBody()){
                                 if (ip.getContentType().indexOf("text/plain") != -1){
                                     totalsize = totalsize + ip.getSize();
                                 } else{
                                     allParts.remove(y);
                                 }
                             } else {
                                 if (ip.getDisposition().equals(Part.INLINE) &&
                                     ip.getContentType().indexOf("text/plain") != -1){
                                     totalsize = totalsize + ip.getSize();                                     
                                 } else {
                                     allParts.remove(y);
                                 }
                             }                             
                         }
                         
                    }
                    
                } else {
                                        
                    getPartFromSingle(allParts, part, numBytes, getOnlyBody); 
                    
                    lastGottenPart = allParts.size()-1;
                    
                    if (lastGottenPart >= 0){
                        
                        ip = (InternalPart)allParts.get(lastGottenPart);

                        if (getOnlyBody){
                            
                            if (ip.isBody()){
                               break; 
                            } else {
                               if (ip.getDisposition().equals(Part.INLINE) &&
                                   ip.getContentType().indexOf("text/plain") != -1){
                                     // inline without "isBody" set
                                     break;
                               }
                            }
                            
                        } else {
                            
                            if (ip.isBody()){
                                if (ip.getContentType().indexOf("text/html") != -1 && 
                                    bodyAlreadyGotten){
                                    allParts.remove(lastGottenPart);
                                } else {
                                   totalsize = totalsize + ip.getSize();  
                                   bodyAlreadyGotten = true;
                                }
                            } else {
                                if ((totalsize + ip.getSize()) > numBytes){
                                    allParts.remove(lastGottenPart);                                        
                                } else {
                                    totalsize = totalsize + ip.getSize(); 
                                }                                                                                               
                            }
                            
                        }
                        
                    }   
                    
                }  
                
            } // end for (get all parts)
            
        } catch (IOException ioe) {
            throw new EntityException(ioe);
        } catch (MessagingException me) {
            throw new EntityException(me);
        }     
        
        return allParts;
        
    }
    
    /**
     * in the InternalBody there are the following values
     * the body; the content type; the stop value; the body size
     *
     * @param multipart Multipart
     * @param numBytes  int
     * @return InternalBody
     * @throws EntityException
     */
    private void getPartFromMulti(ArrayList out, 
                                  Multipart multipart, 
                                  int numBytes,
                                  boolean getOnlyBody)
      throws EntityException {
        
        try {
            
            int partsCount = multipart.getCount();
            
            for (int i = 0; i < partsCount; i++) {
                getPartFromSingle(out, multipart.getBodyPart(i), numBytes, getOnlyBody);                
            }
            
        } catch (MessagingException me) {
            throw new EntityException(me);
        }     
        
    }
    
    
    /**
     *
     * returns the body of the mail without attachment.
     * the method doesn't include the Part if disposition=Part.ATTACHMENT 
     * or disposition=Part.INLINE
     * <p>
     * output: InternalBody
     * <p> [0] (Body)
     * - string if we have a text/plain
     * - html   if we have a text/html
     * - empty string otherwise
     * <p> [1] (String)
     * - related content type
     * <p> [2] (String)
     * - flag to stop the Multipart loop
     * <p> [3] (int)
     * - size of the body
     *
     * 
     * @todo - manage UTF7 :"UTF7" ; "UNICODE-1-1-UTF-7",
     * "csUnicode11UTF7", "UNICODE-2-0-UTF-7"
     *
     * @param part Part
     * @param numBytes int
     * @return InternalBody
     * @throws EntityException
     */
    private void getPartFromSingle(ArrayList out,
                                   Part part, 
                                   int numBytes,
                                   boolean getOnlyBody)
      throws EntityException {
        
        
        try {
            
            InternalPart ip = new InternalPart(Def.DEFAULT_CONTENT_VALUE_BODY,
                                               Def.DEFAULT_CONTENT_VALUE_CONTENTTYPE,
                                               null,
                                               null,
                                               null,
                                               false,
                                               0);

            out.add(ip);
            
            String      contentType     = null;
            String specifiedCharset     = null;
            String      disposition     = part.getDisposition();
            String      contentTypeOrig = part.getContentType().toLowerCase();
            int         size            = part.getSize();
            DataHandler dh              = part.getDataHandler();

            if (contentTypeOrig.equalsIgnoreCase("message/delivery-status") ||
                contentTypeOrig.equalsIgnoreCase("message/rfc822")) {                
                // do nothing                
            } else {
                
               contentType = getBasicContentType(contentTypeOrig);                
               ContentType ct   = new ContentType(part.getContentType());            
               specifiedCharset = ct.getParameter("charset");

               if (log.isLoggable(Level.FINEST)) {
                   log.finest("Content Type Orig : [" + contentTypeOrig + "]" +
                              "Content Type New : [" + contentType + "]" +
                              "Disposition : [" + disposition + "]" +
                              "specifiedCharset [" + specifiedCharset + "]" +
                              "Content Class [" + part.getClass() + "]");
               }

               if (part instanceof MimeBodyPart || part instanceof Message) {
                   
                   if (disposition == null) {

                      // CONTENT-DISPOSITION = NULL
 
                      if (contentType.equals("text/plain") ||  
                          contentType.equals("text/html") ) {
                          
                          if ("utf-7".equalsIgnoreCase(specifiedCharset) ||
                              "unicode-1-1-utf-7".equalsIgnoreCase(specifiedCharset)) {
                              ip.setPartValue(Def.DEFAULT_CONTENT_VALUE_BODY);
                              ip.setContentType(contentTypeOrig);
                              ip.setIsBody(true);
                              ip.setDataHandler(dh);
                              ip.setSize(0);  
                          } else {
                              ip.setPartValue(part.getContent());
                              ip.setContentType(contentTypeOrig);
                              ip.setIsBody(true);
                              ip.setDataHandler(dh);
                              ip.setSize(size); 
                              if (numBytes != -1){
                                 if (numBytes < ip.getSize()) {
                                    byte[] temp = ip.getPartValue().toString().getBytes();
                                    ip.setPartValue(new String(temp, 0, numBytes));
                                    ip.setSize(numBytes); 
                                 } 
                              }                          
                          }
                          
                     } else if (contentType.equals("image/gif")){
                          
                           ip.setPartValue(part.getContent());
                           ip.setContentType(contentTypeOrig);
                           ip.setDisposition(disposition);
                           ip.setSize(size); 
                           ip.setFileName(part.getFileName());
                           ip.setDataHandler(dh);
                          
                     }    

                  } else if (disposition.equalsIgnoreCase(Part.ATTACHMENT)){
                       
                      // CONTENT-DISPOSITION = ATTACHMENT
                       
                      if (!getOnlyBody){
                          ip.setPartValue(part.getContent());
                          ip.setContentType(contentTypeOrig);
                          ip.setDisposition(disposition);
                          ip.setSize(size); 
                          ip.setFileName(part.getFileName());
                          ip.setDataHandler(dh);
                      }  
                      
                  } else  if (disposition.equalsIgnoreCase(Part.INLINE)) {
                       
                      // CONTENT-DISPOSITION = INLINE
                      if (contentType.equals("text/plain") ||  
                          contentType.equals("text/html") ) {
                          ip.setPartValue(part.getContent());
                          ip.setContentType(contentTypeOrig);
                          ip.setDisposition(disposition);
                          ip.setSize(size); 
                          ip.setFileName(part.getFileName());
                          ip.setDataHandler(dh);                          
                      }
                       
                  }
               }
                
            }

            
        } catch (MessagingException me) {
            throw new EntityException(me);
        } catch (IOException ioe) {
            throw new EntityException(ioe);
        }

    }
    
    
    /**
     * <p>Retrieve the text parts of a message or a multipart.<p>
     *
     * @param part the input object (message or multipart)
     * @return an array with the input object's text parts
     * @throws MessagingException
     */
    private Part[] getTextParts(Object part)  throws MessagingException {
        List partsList;
        String contentType;
        int partsCount;
        Multipart multipart;
        Part[] parts;

        if (part == null) {
            return null;
        }

        partsList = new LinkedList();
        try {
            if (part instanceof Message && 
                ((Message) part).getContent() instanceof Multipart) {
                return getTextParts(((Message) part).getContent());
            }
        }
        catch (IOException e) {
            throw new MessagingException(e.getMessage(), e);
        }

        if (part instanceof Multipart) {
            multipart = (Multipart) part;
            partsCount = multipart.getCount();
            for (int i = 0; i < partsCount; i++) {
                parts = getTextParts(multipart.getBodyPart(i));
                for (int j = 0; j < parts.length; j++) {
                    partsList.add(parts[j]);
                }
            }
        } else {
            contentType = ((Part) part).getContentType();
            if (contentType.toLowerCase().startsWith("text/plain") ||
                contentType.toLowerCase().startsWith("text/html")) {
                partsList.add(part);
            }
        }

        return (Part[]) partsList.toArray(new Part[0]);
    }
    
    
    /**
     * <p>Checks if the given message has text parts formed only of ASCII characters</p>
     *
     * @param message the input message
     * @return <i>true</i> if message is ASCII-only, <i>false</i> otherwise
     * @throws MessagingException
     */
    private boolean isASCIIOnly(Message msg) throws MessagingException {
        Object rawContent;
        Part[] textParts;

        // save the flag
        boolean read      = false;
        boolean forwarded = false;
        boolean replied   = false;
        if (msg != null) {
            read      = msg.isSet(Flags.Flag.SEEN);
            forwarded = false; 
            replied   = msg.isSet(Flags.Flag.ANSWERED);
        }
        
        textParts = getTextParts(msg);
        
        if (textParts == null) {
            throw new MessagingException("No text parts found in message.");
        }
        
        for (int i = 0; i < textParts.length; i++) {
            try {
                rawContent = textParts[i].getContent();
            } catch (IOException e) {
                throw new MessagingException(e.getMessage(), e);
            }

            if (rawContent == null) {
                continue;
            }
            
            if (!isASCIIOnly(rawContent.toString())) {
                return false;
            }
            
        }

        // re-set the flag on the server
        msg.setFlag(Flags.Flag.SEEN, read);
        //msg.setFlag(Flags.Flag.FORWARDED, forwarded);
        msg.setFlag(Flags.Flag.ANSWERED,replied);
        
        
        return true;
    }
    
    
    /**
     * <p>Checks if the given text is formed only of ASCII characters</p>
     *
     * @param text the input text
     * @return <i>true</i> if text is ASCII-only, <i>false</i> otherwise
     */
    private boolean isASCIIOnly(String text) {
        char[] chars;

        if (text == null) {
            return false;
        }

        chars = text.toCharArray();
        for (int i = 0; i < chars.length; i++) {
            if (chars[i] > 126) {
                return false;
            }
        }
        return true;
    }   
    
    /**
     *
     * @param text the complete content type
     * @return the basic content type description String
     */
    private String getBasicContentType(String contentTypeOrig) {
        
        String basicContentType = Def.DEFAULT_CONTENT_VALUE_CONTENTTYPE;
        
        if (contentTypeOrig.length() >= 10) {
            int index        = contentTypeOrig.indexOf(";");
            basicContentType = contentTypeOrig.toLowerCase().substring(0, index);
        }
        
        return basicContentType;
        
    }
    
    
}
