/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.transport;

import java.util.Locale;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Folder;

import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.items.model.DefaultFolder;


/**
 *
 * <p>Interface for Mail Server Wrapper </p>
 *
 * @version $Id: IMailServerWrapper.java,v 1.9 2006/05/31 16:34:15 gbmiglia Exp $
 */
public interface IMailServerWrapper {


    /**
     * opens the connection with the mail server
     *
     *
     * @param outgoingServer String
     * @param outgoingPort String
     * @param incomingServer String
     * @param incomingPort String
     * @param defaultFolder DefaultFolder
     * @param user String
     * @param password String
     * @param isSSL boolean
     * @throws EmailAccessException
     */
    public void openConnection(String  outgoingServer,
                               String  outgoingPort,
                               boolean smtpAuth,
                               String  incomingServer,
                               String  incomingPort,
                               DefaultFolder defaultFolder,
                               String  user,
                               String  password,
                               boolean isSSL,
                               String keystorePath,
                               String keystorePassphrase)
      throws EmailAccessException ;


    /**
     * closes the connection with the mail server
     *
     * @throws EmailAccessException
     */
    public void releaseConnection() throws EmailAccessException;


    /**
     *
     *
     * @param server The hostname of the incoming server.
     * @param incomingPort The port number of the incoming server.
     * @param user The username of the account to be used for the incoming server.
     * @param password The password of the account to be used for the incoming server.
     * @param isSSL Indication whether SSL is to be used.
     * @param keystorePath Path to the keystore file used in SSL certification
     * @param keystorePassphrase Passphrase of the keystore file
     * @throws EmailAccessException
     */
    public void checkUser(String server,
                          String incomingPort,
                          String user,
                          String password,
                          boolean isSSL,
                          String keystorePath,
                          String keystorePassphrase)
     throws EmailAccessException;


    /**
     *
     *
     * @return Session
     */
    public Session getSession();

    /**
     *
     * @return Store
     */
    public Store getStore();

    /**
     *
     * @return DefaultFolder
     */
    public DefaultFolder getDefaultFolder();


    /**
     *
     * @return Folder
     */
    public Folder getMailDefaultFolder();


    /**
     *
     *
     * @return Locale
     */
    public Locale getLocale();

}
