/**
 * Copyright (C) 2005-2006 Funambol
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.email.transport  ;

import java.util.logging.Level;
import java.util.Properties;
import java.security.Security;
import javax.mail.Session;
import com.sun.net.ssl.internal.ssl.Provider;
import com.sun.mail.imap.IMAPFolder;
import javax.mail.MessagingException;

import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.items.model.DefaultFolder;


/**
 *
 * <p>
 * Create a connection with a Mail Server
 * using the IMAP protocol
 * </p>
 *
 * @version $Id: ImapMailServerWrapper.java,v 1.11 2006/05/31 16:34:15 gbmiglia Exp $
 */
public class ImapMailServerWrapper
        extends CommonMailServerWrapper
        implements IMailServerWrapper
{

    // -------------------------------------------------------------- Properties


    // ---------------------------------------------------------- PUBLIC METHODS


    /**
     * opens the connection with the mail server
     *
     *
     * @param outgoingServer SMPT address i.e. 192.168.0.1
     * @param outgoingPort 25 def.
     * @param incomingServer IMAP address i.e. 192.168.0.1
     * @param incomingPort 143 / 993 IMAP def.
     * @param df inbox, outbox, sent items,...
     * @param user String
     * @param password String
     * @param isSSL true/false
     * @throws EmailAccessException
     */
    public void openConnection(String  outgoingServer,
                               String  outgoingPort,
                               boolean smtpAuth, 
                               String  incomingServer,
                               String  incomingPort,
                               DefaultFolder df,
                               String  user,
                               String  password,
                               boolean isSSL, 
                               String keystorePath,
                               String keystorePassphrase)
      throws EmailAccessException {

      if (log.isLoggable(Level.FINEST)) {
          log.finest("\nCONNECTION PARAMs"+
                     "\nout          " + outgoingServer+
                     "\nport         " + outgoingPort+
                     "\nsmtpauth     " + smtpAuth+
                     "\nin           " + incomingServer+
                     "\nport         " + incomingPort+
                     "\nprotocol     " + this.getClass().getName()+
                     "\nuser         " + user+
                     "\nSSL          " + isSSL+
                     "\nkeystore     " + keystorePath+
                     "\npassphrase   " + keystorePassphrase);
      }

      // containe for properties
      Properties props = new Properties();

      // provider for a SSL connection
      Provider provider = null;


      if (isSSL) {
        this.setConnectionSSL(props,
                              outgoingServer,
                              outgoingPort,
                              smtpAuth,
                              incomingServer,
                              incomingPort,
                              provider,
                              keystorePath,
                              keystorePassphrase);
      } else {
        this.setConnectionNormal(props,
                                 outgoingServer,
                                 outgoingPort,
                                 smtpAuth,
                                 incomingServer,
                                 incomingPort);
      }

      try{
          
        if (smtpAuth){  
           ConnectorAuthenticator auth = new ConnectorAuthenticator();
           auth.setUserName(user);
           auth.setPassword(password);
           this.session = Session.getInstance(props, auth);
        } else {
           this.session = Session.getInstance(props, null);
        }
                  
        //this.session.setDebug(true);
        this.store = session.getStore();
        this.store.connect(incomingServer, user, password);
        
      } catch (Exception e) {
        log.severe("Error Opening Connection Store: " + e);
        log.throwing(this.getClass().getName(), "openConnection", e);
	    throw new EmailAccessException("Error Opening Connection Store (" + e.getMessage() + ")");
      }

      try {
          this.mailDefaultFolder  = store.getDefaultFolder();
          this.defaultFolder      = df;
      } catch (Exception e) {
          log.severe("Error getting Folders: " + e);
          log.throwing(this.getClass().getName(), "openConnection", e);
          throw new EmailAccessException("Error getting Folders (" +
                                         e.getMessage() + ")");
      }

      if (log.isLoggable(Level.FINEST)) {
          log.finest("IMAP end opening Connection");
      }

    }


    /**
     * closes the connection with the mail server
     *
     * @throws EmailAccessException
     */
    public void releaseConnection() throws EmailAccessException {
        try {
            this.store.close();
            this.store = null;
            this.session = null;
        } catch (Exception e) {
     	    throw new EmailAccessException("Error Closing Store " + e.toString());
        }

    }

    /**
     *
     * returns Inbox Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getInboxFolder() throws EmailAccessException {
       IMAPFolder inbox;
       try {
           inbox = (IMAPFolder)mailDefaultFolder.getFolder(
                                             defaultFolder.getInboxName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting Inbox Folder " +
                                          me.toString());
       }
       return inbox;
    }


    /**
     *
     * returns Outbox Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getOutboxFolder() throws EmailAccessException {
       IMAPFolder outbox;
       try {
           outbox = (IMAPFolder)mailDefaultFolder.getFolder(
                                              defaultFolder.getOutboxName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting outbox Folder " +
                                          me.toString());
       }
       return outbox;
    }
    
    
    /**
     *
     * returns Sent Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getSentFolder() throws EmailAccessException {
       IMAPFolder sent;
       try {
           sent = (IMAPFolder)mailDefaultFolder.getFolder(
                                              defaultFolder.getSentName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting sent Folder " +
                                          me.toString());
       }
       return sent;
    }

    /**
     *
     * returns Drafts Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getDraftsFolder() throws EmailAccessException {
       IMAPFolder drafts;
       try {
           drafts = (IMAPFolder)mailDefaultFolder.getFolder(
                                              defaultFolder.getDraftName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting drafts Folder " +
                                          me.toString());
       }
       return drafts;
    }
 
    /**
     *
     * returns Trash Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getTrashFolder() throws EmailAccessException {
       IMAPFolder trash;
       try {
           trash = (IMAPFolder)mailDefaultFolder.getFolder(
                                              defaultFolder.getTrashName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting trash Folder " +
                                          me.toString());
       }
       return trash;
    }

    //---------------------------------------------------------- PRIVATE METHODS

   /**
    *
    *
    * @param props Properties
    * @param outgoingServer String
    * @param outgoingPort String
    * @param incomingServer String
    * @param incomingPort String
    * @param provider Provider
    */
   protected void setConnectionSSL(Properties props,
                                   String outgoingServer,
                                   String outgoingPort,
                                   boolean smtpAuth,
                                   String incomingServer,
                                   String incomingPort,
                                   Provider provider,
                                   String keystorePath,
                                   String keystorePassphrase)
   {

        if (log.isLoggable(Level.FINEST)) {
           log.finest("IMAP set an SSL connection");
        }

        props.setProperty("mail.smtp.host" , outgoingServer);
        props.setProperty("mail.smtp.port" , outgoingPort);
        props.setProperty("mail.transport.protocol" , "smtp");

        if (smtpAuth){
           props.setProperty("mail.smtp.auth", "true");
        }
        
        props.setProperty("mail.imap.host" , incomingServer);
        props.setProperty("mail.imap.port" , incomingPort);

        props.setProperty("mail.store.protocol", "imap");

        System.setProperty("java.protocol.handler.pkgs",
                                  "com.sun.net.ssl.internal.www.protocol");
        System.setProperty("javax.net.ssl.trustStore", keystorePath);
       System.setProperty("javax.net.ssl.trustStorePassword", keystorePassphrase);

        provider = new Provider();
	    Security.addProvider(provider);

        // set SSL parameters
    	props.setProperty("mail.imap.socketFactory.class",
                                 "javax.net.ssl.SSLSocketFactory");
    	props.setProperty("mail.imap.socketFactory.fallback",
                                 "false");
    	props.setProperty("mail.imap.socketFactory.port",
                                 incomingPort);

    }

    /**
     *
     *
     *
     * @param props Properties
     * @param outgoingServer String
     * @param outgoingPort String
     * @param incomingServer String
     * @param incomingPort String
     */
    protected void setConnectionNormal(Properties props,
                                       String outgoingServer,
                                       String outgoingPort,
                                       boolean smtpAuth,
                                       String incomingServer,
                                       String incomingPort){

        if (log.isLoggable(Level.FINEST)) {
           log.finest("IMAP set a normal connection");
        }

        props.setProperty("mail.smtp.host", outgoingServer);
        props.setProperty("mail.smtp.port", outgoingPort);
        props.setProperty("mail.transport.protocol", "smtp");

        if (smtpAuth){
           props.setProperty("mail.smtp.auth", "true");
        }
        
        props.setProperty("mail.imap.host"    , incomingServer);
        props.setProperty("mail.imap.port"    , incomingPort);

        props.setProperty("mail.store.protocol", "imap");
    }

}
